/*******************************************************************************
*
* File:         s8_demo.c
* Description:  系统测试文件
* Version:      V0.1
* Date:         2020-02-22
* Author:       Tempest
 *******************************************************************************/
#include "s8_demo.h"
#include "ArmCM0.h"
#include "tau_device_datatype.h"
#include "tau_log.h"
#include "tau_operations.h"
#include "tau_common.h"
#include "tau_delay.h"
#include "hal_dsi_rx_ctrl.h"
#include "hal_dsi_tx_ctrl.h"
#include "hal_swire.h"
#include "hal_timer.h"
#include "hal_system.h"
#include "hal_gpio.h"
#include "test_cfg_global.h"
#include "hal_pwr.h"
#include "hal_pwm.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif
#define LOG_TAG "s8_demo"

#if _DEMO_S8_EN

/******************** FEATURE开关 ********************/
#define TOUCH_ENABLE                   false     /* Touch转换开关 */
#define RX_START_WITHOUT_RST           false    /* 不等待AP RESET直接启动RX,仅作为调试使用 */
#define RX_WAIT_TEAR_ON                false     /* 等待AP_TEAR_ON */
#define RX_RESOLUTION_CHANGE_ENABLE    true     /* 支持分辨率切换开关, AP存在分辨率切换时需要打开 */
#define RX_READ_HW_ACK                 true     /* AP DCS读命令使用硬件回复 */
#define TX_START_AFTER_APRST           false    /* 等待AP_reset 后做Panel初始化, 用于热拔插电源不稳定导致初始化失败 */
#define TX_USE_CMD_MODE                false    /* command mode输出开关 配置为True时需要把TD TE与 AP TE 接一起*/
#define TX_CMD_MODE_WITHOUT_TE         false    /* 屏端TE直接输出给AP,Scaler不看TE信号,AP每输入一帧输出一帧(C2C 60Hz->60Hz) */
#define AP_SWIRE_OUTPUT                true     /* swire信号输出,OLED项目配置 */
#define ANALOG_PWM_OUTPUT              false    /* 模拟PWM 调光开关 */
#define SHARE_FLASH_ENABLE             false    /* 共享flash开关 */
/*****************************************************/

/******************** 输出屏幕选择 ********************/
#define AMOLED_NT37280                 0                   /* 4lane FHD Panel */
#define LCD_NT36525B                   0                   /* 3lane HD  Panel */
#define OLED_NT37705_TM678             1
/*****************************************************/

/****************** 系统相关参数配置 ******************/
#define MAIN_POWER_SELECT              PWR_SEL_VCC             /* 主供电电源选择 */
#define SLEEP_MODE_POWER               PWR_SLEEP_IN_TP18           /* 息屏电源选择 */
#define SLEEP_MODE_SELECT              PWR_NORMAL_SLEEP_MODE   /* sleep mode 配置 */
#define SLEEP_MODE_DEBUG               false                   /* sleep mode debug开关 */
#define SWIRE_DEFAULT_PULSE            42 // 53                      /* SWIRE 波形配置 */
#define PWM_FREQUENCY                  30000                   /* PWM输出频率30Khz */
#define PWM_DUTY_STEP                  255                     /* PWM调光阶数255阶 */
/*****************************************************/

/****************** 自定义配置 ***********************/
#define PANEL_INIT_CODE_ARRAY          1                        /* 初始化使用数组模式配置 */ 
#define USE_9E_AS_PPS_UPDATE           1                        /* pps更新特殊指令 */
#define TEST_TX_VPG                    0                        /* vpg测试 */
/*****************************************************/



/********************RX 基本参数配置*******************/
//S8 MIPI数据信息
/* 输入分辨率 */
#define INPUT_WIDTH             1440//1080//1440
#define INPUT_HEIGHT            3200//2400//3200
/* 输入 MIPI lane rate,需要正确配置,可50M step调整 */
#define INPUT_MIPI_LANE_RATE    1500000000
/* 输入图像格式 */
#define INPUT_COLOR_MODE        DSI_RGB10_10_10
/* 输出数据格式(DSI_DATA_CMD_MODE , DSI_DATA_VIDEO_MODE) */
#define INPUT_DATA_MODE         DSI_DATA_CMD_MODE
/* 输入mipi lane数量(DSI_RX_LANE_x     x为1-4) */
#define INPUT_MIPI_LANE_NUM     DSI_LANE_4
/* 输入为video mode 时数据格式 */
#define INPUT_VIDEO_MODEL       DSI_NONBURST_EVENT
/* 输入虚拟通道(0-3) */
#define INPUT_VC                DSI_VC_0
/* 输入的帧率(60/90/120/144Hz) */
#define INPUT_FRAME_RATE        DSI_FRAME_RATE_60HZ
/* 输入数据是否DSC压缩 */
#define INPUT_COMPRESS          true
/*****************************************************/

/********************TX 基本参数配置*******************/
#if AMOLED_NT37280
//almoled NT37280输出MIPI数据信息
/* 输出分辨率配置 */
#define OUTPUT_WIDTH            1080
#define OUTPUT_HEIGHT           2280
/* 输出虚拟通道(0-3) */
#define OUTPUT_VC               DSI_VC_0
/* 输出mipi lane数量(DSI_RX_LANE_x     x为1-4) */
#define OUTPUT_LANE_NUMBER      DSI_LANE_4
/* 输出数据格式(DSI_DATA_CMD_MODE , DSI_DATA_VIDEO_MODE) */

#if TX_USE_CMD_MODE
#define OUTPUT_DATA_MODE        DSI_DATA_CMD_MODE
#else
#define OUTPUT_DATA_MODE        DSI_DATA_VIDEO_MODE
#endif
/* 输出为video mode时的数据格式 */
#define OUTPUT_VIDEO_MODEL      DSI_BURST_MODE
/* 输出 VSA */
#define OUTPUT_VSA              4
/* 输出 VBP */
#define OUTPUT_VBP              28
/* 输出 VBP */
#define OUTPUT_VFP              10
/* 输出 VSA */
#define OUTPUT_HSA              8
/* 输出 HBP */
#define OUTPUT_HBP              16
/* 输出 HFP */
#define OUTPUT_HFP              36
/* 初始化模式命令传输类型 LP/HS */
#define TX_INIT_TYPE            DSI_CMD_TX_LP

#elif LCD_NT36525B
/* 输出分辨率配置 */
#define OUTPUT_WIDTH            720
#define OUTPUT_HEIGHT           1520
/* 输出虚拟通道(0-3) */
#define OUTPUT_VC               DSI_VC_0
/* 输出mipi lane数量(DSI_RX_LANE_x     x为1-4) */
#define OUTPUT_LANE_NUMBER      DSI_LANE_3
/* 输出数据格式(DSI_DATA_CMD_MODE , DSI_DATA_VIDEO_MODE) */
#define OUTPUT_DATA_MODE        DSI_DATA_VIDEO_MODE
/* 输出为video mode 的数据格式 */
#define OUTPUT_VIDEO_MODEL      DSI_BURST_MODE
/* 输出 VSA */
#define OUTPUT_VSA              30
/* 输出 VBP */
#define OUTPUT_VBP              20
/* 输出 VBP */
#define OUTPUT_VFP              4
/* 输出 VSA */
#define OUTPUT_HSA              30
/* 输出 HBP */
#define OUTPUT_HBP              30
/* 输出 HFP */
#define OUTPUT_HFP              30
/* 初始化模式命令传输类型 LP/HS */
#define TX_INIT_TYPE            DSI_CMD_TX_LP

#elif OLED_NT37705_TM678
//almoled NT37280输出MIPI数据信息
/* 输出分辨率配置 */
#define OUTPUT_WIDTH            1080
#define OUTPUT_HEIGHT           2400
/* 输出虚拟通道(0-3) */
#define OUTPUT_VC               DSI_VC_0
/* 输出mipi lane数量(DSI_RX_LANE_x     x为1-4) */
#define OUTPUT_LANE_NUMBER      DSI_LANE_4
/* 输出数据格式(DSI_DATA_CMD_MODE , DSI_DATA_VIDEO_MODE) */
#if TX_USE_CMD_MODE
#define OUTPUT_DATA_MODE        DSI_DATA_CMD_MODE
#else
#define OUTPUT_DATA_MODE        DSI_DATA_VIDEO_MODE
#endif
/* 输出为video mode时的数据格式 */
#define OUTPUT_VIDEO_MODEL      DSI_BURST_MODE
/* 输出 VSA */
#define OUTPUT_VSA              2//4 //8//4
/* 输出 VBP */
#define OUTPUT_VBP              8//10//12
/* 输出 VBP */
#define OUTPUT_VFP              32//20//56//72
/* 输出 VSA */
#define OUTPUT_HSA              8//10//8
/* 输出 HBP */
#define OUTPUT_HBP              60//30//12//92
/* 输出 HFP */
#define OUTPUT_HFP              60//120//100
/* 初始化模式命令传输类型 LP/HS */
#define TX_INIT_TYPE            DSI_CMD_TX_LP
#endif
/******************************************************/

#if TOUCH_ENABLE
#include "app_tp_transfer.h"
#include "app_tp_for_custom_s8.h"
#endif

/* 全局handle */
static hal_dsi_rx_ctrl_handle_t *g_rx_ctrl_handle = NULL;
static hal_dsi_tx_ctrl_handle_t *g_tx_ctrl_handle = NULL;

/* 屏初始化完成标志位 */
static bool panel_display_done = false;
static bool sg_system_resume = false;
static bool sg_system_suspend = false;
//static bool ap_reg51_value = false;
static bool AP_REG51_7FF = false;

static bool sg_aod_in_flag = false;

#if RX_WAIT_TEAR_ON
static bool sg_ap_set_tear_on = false;
#endif
#if TX_START_AFTER_APRST
static bool sg_tx_start_in_process = false;
#endif


/* AP reset 回调函数声明 */
static void ap_rstn_pull_high_cb(void *data);
static void ap_rstn_pull_down_cb(void *data);
static void app_mipi_rx_start_cb(void *data);

/* 通话息屏 */
#define TOUCH_PHONE_DIS_OFF_PIN             ( IO_PIN_7 )        // 通话熄屏响应管脚

/* soft swire */
#define AVDD_EN                 ( IO_PIN_17 )
#define SWIRE_EN                ( IO_PIN_15 )
#define SWIRE_MAX_NUM           ( 10 )
#define SWIRE_DEFAULT_NUM       ( 53 )
#define SOFT_SWIRE_TIMER        ( TIMER_NUM3 )
uint8_t s_swire_pwm_counter     = 0;

/*************************DCS 命令处理函数 BEGIN*************************/
#if RX_READ_HW_ACK
/**
* @brief  配置AP硬件回读
* @param  none
* @retval none
*/
static void app_set_dcs_hw_ack()
{
    /*X90Pro+*/
    hal_dsi_rx_ctrl_set_auto_ack(g_rx_ctrl_handle, HAL_RX_QRESP_CODE0, DSI_ACK_DT_DCS_SHORT_RESPONSE_1B, 0x0F, 1, 0xC0);
    hal_dsi_rx_ctrl_set_auto_ack(g_rx_ctrl_handle, HAL_RX_QRESP_CODE1, DSI_ACK_DT_DCS_SHORT_RESPONSE_1B, 0xDA, 1, 0x4B);
    hal_dsi_rx_ctrl_set_auto_ack(g_rx_ctrl_handle, HAL_RX_QRESP_CODE2, DSI_ACK_DT_DCS_SHORT_RESPONSE_1B, 0xDB, 1, 0xE0);
    hal_dsi_rx_ctrl_set_auto_ack(g_rx_ctrl_handle, HAL_RX_QRESP_CODE3, DSI_ACK_DT_DCS_SHORT_RESPONSE_1B, 0xDC, 1, 0xE0);
    hal_dsi_rx_ctrl_set_auto_ack(g_rx_ctrl_handle, HAL_RX_QRESP_CODE4, DSI_ACK_DT_DCS_SHORT_RESPONSE_1B, 0x0A, 1, 0x9F);
    hal_dsi_rx_ctrl_set_auto_ack(g_rx_ctrl_handle, HAL_RX_QRESP_CODE5, DSI_ACK_DT_DCS_SHORT_RESPONSE_1B, 0x6E, 1, 0xEF);//打开会使触摸无作用
}
#endif
/**
* @brief  ap 读回调函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_read(uint8_t data_type, uint8_t dcs_cmd, uint8_t param)
{
#if !RX_READ_HW_ACK
    if (dcs_cmd == 0x0F)
    {
        hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
                                     DSI_VC_0,
                                     1, 0xC0);
    }
    else if (dcs_cmd == 0xDA)
    {
        hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
                                     DSI_VC_0,
                                     1, 0x4B);
    }
    else if (dcs_cmd == 0xDB)
    {
        hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
                                     DSI_VC_0,
                                     1, 0xE0);
    }
    else if (dcs_cmd == 0xDC)
    {
        hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
                                     DSI_VC_0,
                                     1, 0xE0);
    }
    else if (dcs_cmd == 0x0F)
    {
        hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
                                     DSI_VC_0,
                                     1, 0xC0);
    }
    else if (dcs_cmd == 0x6E)
    {
        hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
                                     DSI_VC_0,
                                     1, 0xEF);
    }
    else if (dcs_cmd == 0x04)
#else
    if (dcs_cmd == 0x04)
#endif
    {
        hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_LONG_RESPONSE,
                                     DSI_VC_0,
                                     3, 0xD1, 0x40, 0x15);
    }
    else if (dcs_cmd == 0x0A)
    {
        uint32_t return_size = hal_dsi_rx_ctrl_get_max_ret_size(g_rx_ctrl_handle);
        if (return_size == 1)
        {
            hal_dsi_rx_ctrl_send_ack_cmd(g_rx_ctrl_handle,
                                     DSI_ACK_DT_DCS_SHORT_RESPONSE_1B,
                                     DSI_VC_0,
                                     1, 0x9F);
        }
        else
        {
            TAU_LOGD("resp error return_size[%d]!\n", return_size);
        }
    }
    else
    {
        TAU_LOGD("r %x ng need return [%d]\n", dcs_cmd, hal_dsi_rx_ctrl_get_max_ret_size(g_rx_ctrl_handle));
    }
    return true;
}


#if RX_RESOLUTION_CHANGE_ENABLE
/* PPS update callback 用于分辨率切换case */
static bool pps_update_handle(uint8_t *pps, uint8_t size, uint32_t pic_width, uint32_t pic_height)
{
//    /* AVDD 上电, 用于解决息屏开屏PPS不更新问题 */
   if (pic_width != g_rx_ctrl_handle->base_info.src_w || pic_height != g_rx_ctrl_handle->base_info.src_h)
    {
        /* PPS Update 且分辨率发生变化 */
        g_rx_ctrl_handle->base_info.src_w = pic_width;
        g_rx_ctrl_handle->base_info.src_h = pic_height;
        /* 注意部分基板更新PPS前不发 Compression Mode Command的情况 */
        g_rx_ctrl_handle->compress_en     = true;  // 压缩保持一致
        g_tx_ctrl_handle->base_info.src_w = pic_width;
        g_tx_ctrl_handle->base_info.src_h = pic_height;
        /* 拿掉了PPS_PREINIT预置函数，无影响 */
        hal_dsi_rx_ctrl_toggle_resolution(g_rx_ctrl_handle);
// TAU_LOGD("resolution update w[%d] h[%d] compress[%d]\n", pic_width, pic_height, g_rx_ctrl_handle->compress_en);       
    }
    return true;
}
#endif

/**
* @brief  ap display on处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_display_on(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    TAU_LOGD("disp on \n");
    return true;
}

/**
* @brief  ap display off处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_display_off(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
 //   TAU_LOGD("disp off %d\n", panel_display_done);
    return true;
}

/**
* @brief  ap enter sleep mode处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_enter_sleep_mode(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
//		sg_system_suspend = true;
    hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x28);
		delayMs( 20 );
    hal_gpio_init_output( IO_PIN_17, IO_LVL_LOW ); // SWIRE close
    delayMs( 20 );
    hal_gpio_init_output( IO_PIN_15, IO_LVL_LOW ); // AVDD close
    delayMs( 10 );
    hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x10);
 //   TAU_LOGD("enter sleep mode \n");
		hal_gpio_set_ap_reset_int(ENABLE, ap_rstn_pull_down_cb, DETECT_LOW_LVL);
    return true;
}

/**
* @brief  ap exit sleep mode处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_exit_sleep_mode(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
#if AP_SWIRE_OUTPUT
    /* AVDD 上电, 用于解决息屏开屏PPS不更新问题 */
    hal_gpio_set_output_data(IO_PAD_AP_PWMEN, IO_LVL_HIGH);
#endif

#if TX_START_AFTER_APRST
    if (panel_display_done == false)
    {
        sg_tx_start_in_process = true;
    }
#endif

    TAU_LOGD("exit sleep mode \n");

    return true;
}

/**
* @brief  调光处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_backlight(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
#if 1 // video mode
        uint16_t rd_51_val=6;
        uint32_t value = 0;
    
        rd_51_val = ( ( dcs_packet->packet_param[0] << 8 ) | ( dcs_packet->packet_param[1] ) );
        
        ///////////////////////////////////////////////////////////////////////////////
        // AP 0x06~0x7FF 映射到 0x20~0xDBB
        if( rd_51_val ==0x7FF )
        {
            AP_REG51_7FF =  true;
            value =0xDBB;
        }
        else if( rd_51_val==0x314)
        {
            /* AOD 调光 */
            value = 0xDBB;
            AP_REG51_7FF =  false;
        }
        else  if(rd_51_val == 0x21F)
        {   
            if(AP_REG51_7FF ==  true)
                {
                  value =0xDBB;
                }else
                {
                    value = 0x3B4;
                    
                }
                AP_REG51_7FF =  false;
        }
        else if( (rd_51_val>=0) && (rd_51_val<6))
        {
            /* AOD 调光 */
						if(sg_aod_in_flag)
						{
							 value = 0x100;  // 0x40
							 AP_REG51_7FF =  false;
						}
           
        }
       else
        {       
            value = (rd_51_val-6)*3483/2041 +0x20;
            AP_REG51_7FF =  false;
        }
						
       if(value>0)
				 hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0x51, value >> 8, value & 0x00FF);
        
 //      TAU_LOGD( "AP_51:[0x%04X], IC_51:[0x%04X] \n", rd_51_val, value );
#endif
        return true;
}

static bool ap_dcs_set_AOD_53(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
	if(dcs_packet->packet_param[0] == 0x24)
	{
		sg_aod_in_flag = true;
	}
	else
	{
		sg_aod_in_flag = false;
	}
	//TAU_LOGD( "AOD_53:[0x%02x],[0x%02X] \n", dcs_packet->packet_param[0], dcs_packet->packet_param[1] );
	return true;
}

#if RX_WAIT_TEAR_ON
/**
* @brief  ap set tear on 处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_tear_on(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    TAU_LOGD("ap_set_tear_on\n");
    sg_ap_set_tear_on = true;
    if (panel_display_done == false)
    {
        TAU_LOGD("gen a tear\n");
        hal_dsi_tx_ctrl_gen_a_tear_signal();
    }
    else
    {
        hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_60HZ_MODE);
    }
    return true;
}

/**
* @brief  ap set tear off 处理函数
* @param  handler：rx_ctrl_handle
*         dcs_packet: dcs 命令
* @retval true/false
*/
static bool ap_dcs_set_tear_off(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    TAU_LOGD("ap_set_tear_off \n");
    sg_ap_set_tear_on = false;
    hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_USER_MODE);
    return true;
}
#endif

/**
* @brief  护眼模式回调函数
* @param  rx_ctrl_handle: dsi rx handle;
*         dcs packet: dcs_packet
* @retval true/false
*/
static bool ap_dcs_set_eye_protection_mode(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
    uint8_t panel_ccm_en = dcs_packet->packet_param[0x00];
    uint16_t   panel_r = 256;
    uint16_t   panel_b = 256;
    /* CCM adjust for display in eye-shield mode */
    if (panel_ccm_en == 0x00)
    {
        panel_r = 256;
        panel_b = 200;
    }  /* CCM adjust for display in self-adaption mode */
    else if (panel_ccm_en == 0x01)
    {
        /* 不调节对角线G值防止锁屏界面出现水印，gg和rr相对，因此考虑对角线R值调节 自适应显示下0xFC/0xFF为数据包R/G最大特征值，因此去基线值0xF0*/
        panel_r = 2 * (dcs_packet->packet_param[0x31] - dcs_packet->packet_param[0x33]) + 0xF0;
        /* 增加变化量提升调节可见效果, 自适应显示下0xFC为数据包B最大特征值*/
        panel_b = 2 * dcs_packet->packet_param[0x35] - 0xFC;
    }

    /* CCM control red, green, blue */
    ccm_coef_t  coef ;
    coef.coef_c00 = panel_r;
    coef.coef_c01 = 0;
    coef.coef_c02 = 0;
    coef.coef_c10 = 0;
    coef.coef_c11 = 256;
    coef.coef_c12 = 0;
    coef.coef_c20 = 0;
    coef.coef_c21 = 0;
    coef.coef_c22 = panel_b;
    hal_dsi_tx_ctrl_set_ccm(&coef);
    return true;
}

/*************************DCS 命令处理函数 END*************************/
#if USE_9E_AS_PPS_UPDATE
static bool ap_update_pps_9E(hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet)
{
//    TAU_LOGD("9E Start\r\n");
    uint32_t pic_width, pic_height;
    unsigned char s_pps[128];
    
	pic_height = (dcs_packet->packet_param[6] << 8) + dcs_packet->packet_param[7];
	pic_width = (dcs_packet->packet_param[8] << 8) + dcs_packet->packet_param[9];
	
	if (pic_width != g_rx_ctrl_handle->base_info.src_w || pic_height != g_rx_ctrl_handle->base_info.src_h)	
	{
	   for (int i = 0; i < dcs_packet->param_length; i++)
	    {
	        s_pps[i] = dcs_packet->packet_param[i];
	    }
		// PPS Update 且分辨率发生变化 //
        g_rx_ctrl_handle->base_info.src_w = pic_width;
        g_rx_ctrl_handle->base_info.src_h = pic_height;
        // 注意部分基板更新PPS前不发 Compression Mode Command的情况 //
        g_tx_ctrl_handle->base_info.src_w = pic_width;
        g_tx_ctrl_handle->base_info.src_h = pic_height;
        hal_dsi_rx_ctrl_pre_init_pps(g_rx_ctrl_handle, s_pps, 128);
        hal_dsi_rx_ctrl_toggle_resolution(g_rx_ctrl_handle);
        // TAU_LOGD(" UpDate resolution \n");
	}
    return true;	
}
#endif    

static bool ap_set_framerate( hal_dsi_rx_ctrl_handle_t *handler, hal_dcs_packet_t *dcs_packet )
{
    /* 切分辨率：120Hz:0 60Hz 1或3， 原：120：0， 60：010 或 03 原理：滤除1后面的0，1、3、为60Hz， 0直接切120Hz */
    
/*    static unsigned char FrameRateBackUp = 0xFF;
    
    if(FrameRateBackUp != dcs_packet->packet_param[ 0 ])
		{
			FrameRateBackUp = dcs_packet->packet_param[ 0 ];
			if(FrameRateBackUp == 0x00)
			{
				hal_dsi_tx_ctrl_set_tear_mode( g_tx_ctrl_handle, TE_60HZ_MODE );
			}
			TAU_LOGI( "FrameRate:%d %d\n", dcs_packet->packet_param[ 0 ], dcs_packet->packet_param[ 1 ]);
		}
		
 //   TAU_LOGI( "FrameRate:%d %d\n", dcs_packet->packet_param[ 0 ], dcs_packet->packet_param[ 1 ]);
*/		
//    if( ( 1 == dcs_packet->packet_param[ 0 ] ) || ( 3 == dcs_packet->packet_param[ 0 ] ) )
//    {
//        /* 收到1或者3，则当前需要切换到60Hz */
        hal_dsi_tx_ctrl_set_tear_mode( g_tx_ctrl_handle, TE_60HZ_MODE );
//        TAU_LOGD(" 60Hz \n");
//    }
//    else if( 1 == FrameRateBackUp )
//    {
//        /* 不执行任何操作，占位 */
//        TAU_LOGD(" None\n");
//    }
//    else if( 0 == dcs_packet->packet_param[ 0 ] )
//    {
//        /* 其它情况：120Hz，正常执行帧率切换 */
//        hal_dsi_tx_ctrl_set_tear_mode( g_tx_ctrl_handle, TE_120HZ_MODE );
//        TAU_LOGD(" 120Hz \n");
//    }
//    
//    FrameRateBackUp = dcs_packet->packet_param[ 0 ]; /* 更新变量，主要是记录在60Hz收到 0 1 0的时候 */
    return true; 
}

/* 客制化DCS command 处理函数表格 */
static const hal_dcs_execute_entry_t g_cus_rx_dcs_execute_table[] =
{
    {DCS_SET_DISPLAY_ON,    ap_dcs_set_display_on,          true},
    {DCS_SET_DISPLAY_OFF,   ap_dcs_set_display_off,         true},
    {0x51,                  ap_dcs_set_backlight,           false},
		{0x53,                  ap_dcs_set_AOD_53,           		true},
    {DCS_ENTER_SLEEP_MODE,  ap_dcs_set_enter_sleep_mode,    true},
    {DCS_EXIT_SLEEP_MODE,   ap_dcs_set_exit_sleep_mode,     true},
#if RX_WAIT_TEAR_ON
    {DCS_SET_TEAR_ON,       ap_dcs_set_tear_on,             true},
    {DCS_SET_TEAR_OFF,      ap_dcs_set_tear_off,            true},
#endif
    {0x9E,                  ap_update_pps_9E,               true},
    {0x60,                  ap_set_framerate,               true},
    {0,                     NULL,                           false} //{0,NULL,false} 数组最后一个固定成员，作为table结尾的判断标准
};

/**
* @brief  panel reset
* @param  none
* @retval none
*/
static void app_tx_panel_reset(void)
{
#if SHARE_FLASH_ENABLE
    hal_flash_share_mode(true);
#endif
    hal_gpio_init_output(IO_PAD_TD_RSTN, IO_LVL_HIGH);
    delayMs(10);
    hal_gpio_init_output(IO_PAD_TD_RSTN, IO_LVL_LOW);
    delayMs(10);
    hal_gpio_init_output(IO_PAD_TD_RSTN, IO_LVL_HIGH);
    delayMs(30);
}

#if PANEL_INIT_CODE_ARRAY
static void send_panel_init_code(uint32_t size, uint8_t* data)
{
    uint32_t data_offeset = 0;
    uint8_t data_type;
    uint8_t vc;
    uint8_t data_size;
    uint8_t* p_data;

    while(data_offeset < size)
    {
        data_type = data[data_offeset];
        vc = data[data_offeset + 1];
        data_size = data[data_offeset + 2];
        p_data = &data[data_offeset + 3];
        hal_dsi_tx_ctrl_write_array_cmd(data_type, vc, data_size, p_data);
        data_offeset = data_offeset + data_size + 3;
        delayUs( 100 );
    }
}
#endif

#if OLED_NT37705_TM678
const uint8_t panel_init_code[] =
{
   //update@2023.11.08 
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0xB0,0x00,
	0x39,0,2,0x6F,0x01,
	0x39,0,3,0xB9,0x04,0x38,
	0x39,0,3,0xBD,0x09,0x60,
	0x39,0,8,0xBA,0x00,0x66,0x00,0x10,0x09,0xB0,0x10,
	0x39,0,2,0x6F,0x07,
	0x39,0,8,0xBA,0x00,0x66,0x00,0x10,0x03,0x50,0x10,
	0x39,0,2,0x6F,0x0E,
	0x39,0,8,0xBA,0x00,0xCC,0x00,0x10,0x00,0x20,0x10,
	0x39,0,2,0x6F,0x15,
	0x39,0,8,0xBA,0x00,0x66,0x00,0x10,0x00,0x20,0x10,
	0x39,0,8,0xBB,0x00,0x66,0x00,0x10,0x00,0x20,0x70,
	0x39,0,5,0xBE,0x40,0x47,0x00,0x00,
	0x39,0,11,0xC1,0x00,0xB4,0x00,0xB4,0x00,0xB4,0x00,0xB4,0x00,0xB4,
	0x39,0,2,0x6F,0x2C,
	0x39,0,2,0xCB,0x30,
	0x39,0,3,0xB2,0x09,0x40,
	0x39,0,2,0x6F,0x04,
	0x39,0,2,0xB2,0x22,
	0x39,0,2,0x6F,0x05,
	0x39,0,3,0xB2,0x1E,0x1E,
	0x39,0,2,0x6F,0x11,
	0x39,0,4,0xB2,0x05,0x03,0x42,
	0x39,0,2,0x6F,0x15,
	0x39,0,2,0xB2,0x03,
	0x39,0,2,0x6F,0x18,
	0x39,0,3,0xB2,0x09,0xC0,
	0x39,0,2,0x6F,0x1A,
	0x39,0,3,0xB2,0x3F,0xF8,
	0x39,0,31,0xB3,0x00,0x05,0x00,0x1A,0x00,0x1A,0x00,0x3D,0x00,0x3D,0x00,0x83,0x00,0x83,0x01,0x57,0x01,0x57,0x02,0x70,0x02,0x70,0x02,0x71,0x02,0x71,0x05,0x78,0x05,0x78,
	0x39,0,2,0x6F,0x1E,
	0x39,0,15,0xB3,0x0D,0xBB,0x0D,0xBB,0x0F,0xFF,0x0F,0xFF,0x0F,0xFF,0x0F,0xFF,0x0F,0xFF,
	0x39,0,2,0x6F,0x2C,
	0x39,0,9,0xB3,0x00,0x05,0x09,0x56,0x09,0x56,0x0F,0xFE,
	0x39,0,31,0xB4,0x13,0x20,0x12,0x48,0x11,0xB8,0x11,0xB8,0x10,0xB0,0x10,0xB0,0xE,0xA0,0xE,0xA0,0x8,0x70,0x8,0x70,0x00,0x48,0x00,0x48,0x00,0x18,0x00,0x18,0x00,0x18,
	0x39,0,2,0x6F,0x1E,
	0x39,0,17,0xB4,0x00,0x18,0x00,0x18,0x00,0x18,0x00,0x18,0x00,0x18,0x00,0x18,0x00,0x18,0x00,0x18,
	0x39,0,2,0x6F,0x2E,
	0x39,0,31,0xB4,0xC,0xC0,0xC,0x30,0xB,0xD0,0xB,0xD0,0xB,0x20,0xB,0x20,0x9,0xC0,0x9,0xC0,0x5,0xA0,0x5,0xA0,0x00,0x30,0x00,0x30,0x00,0x10,0x00,0x10,0x00,0x10,
	0x39,0,2,0x6F,0x4C,
	0x39,0,17,0xB4,0x00,0x10,0x00,0x10,0x00,0x10,0x00,0x10,0x00,0x10,0x00,0x10,0x00,0x10,0x00,0x10,
	0x39,0,2,0x6F,0x5C,
	0x39,0,31,0xB4,0x9,0x90,0x9,0x24,0x8,0xDC,0x8,0xDC,0x8,0x58,0x8,0x58,0x7,0x50,0x7,0x50,0x4,0x38,0x4,0x38,0x00,0x24,0x00,0x24,0x00,0xC,0x00,0xC,0x00,0xC,
	0x39,0,2,0x6F,0x7A,
	0x39,0,17,0xB4,0x00,0xC,0x00,0xC,0x00,0xC,0x00,0xC,0x00,0xC,0x00,0xC,0x00,0xC,0x00,0xC,
	0x39,0,2,0x6F,0xB8,
	0x39,0,11,0xB4,0x09,0x90,0x08,0x90,0x06,0xB0,0x06,0xB0,0x04,0xC0,
	0x39,0,2,0x6F,0x13,
	0x39,0,3,0xC0,0x00,0xAC,
	0x39,0,2,0x6F,0x1F,
	0x39,0,2,0xC0,0x08,
	0x39,0,2,0x6F,0x0B,
	0x39,0,3,0xC0,0x24,0x09,
	0x39,0,2,0x6F,0x0D,
	0x39,0,5,0xC0,0x00,0x00,0x00,0x00,
	0x39,0,3,0xE4,0x01,0x40,
	0x39,0,2,0x6F,0x03,
	0x39,0,4,0xC0,0x21,0x43,0x05,
	0x39,0,2,0xB5,0x80,
	0x39,0,2,0x6F,0x02,
	0x39,0,2,0xB5,0x2B,
	0x39,0,2,0x6F,0x06,
	0x39,0,2,0xB5,0x7F,
	0x39,0,2,0x6F,0x07,
	0x39,0,6,0xB5,0x00,0x28,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x0C,
	0x39,0,6,0xB5,0x00,0x2A,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x11,
	0x39,0,3,0xB5,0x28,0x28,
	0x39,0,2,0x6F,0x18,
	0x39,0,3,0xB5,0x0D,0x19,
	0x39,0,2,0x6F,0x1D,
	0x39,0,3,0xB5,0x15,0x1A,
	0x39,0,2,0x6F,0x23,
	0x39,0,2,0xB5,0x2B,
	0x39,0,2,0x6F,0x27,
	0x39,0,3,0xB5,0x0D,0x0D,
	0x39,0,2,0x6F,0x2C,
	0x39,0,2,0xB5,0x03,
	0x39,0,2,0x6F,0x2D,
	0x39,0,2,0x6F,0x44,
	0x39,0,2,0x6F,0x01,
	0x39,0,2,0xB6,0x19,
	0x39,0,2,0x6F,0x0A,
	0x39,0,2,0xCA,0x22,
	0x39,0,17,0xC6,0x44,0x44,0x44,0x43,0x77,0x77,0x77,0x73,0x77,0x77,0x77,0x73,0x77,0x77,0x77,0x73,
	0x39,0,2,0x6F,0x20,
	0x39,0,9,0xC6,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,
	0x39,0,3,0xC5,0x44,0x44,
	0x39,0,2,0x6F,0x0C,
	0x39,0,2,0xC3,0x00,
	0x39,0,2,0x6F,0x02,
	0x39,0,11,0xC3,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0xC0,0x04,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0x6F,0x07,
	0x39,0,4,0xCA,0x08,0x06,0x37 ,
	0x39,0,2,0xE2,0x40,
	0x39,0,2,0x6F,0x57,
	0x39,0,6,0xE2,0xFF,0xFF,0xFF,0xFF,0xFF,
	0x39,0,2,0x6F,0x37,
	0x39,0,5,0xE2,0x80,0x04,0x00,0x08,
	0x39,0,2,0x6F,0x0B,
	0x39,0,2,0xC2,0x11,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x01,
	0x39,0,3,0xB0,0x2F,0x28,
	0x39,0,3,0xB1,0x25,0x25,
	0x39,0,5,0xB2,0xAA,0x02,0x55,0x02,
	0x39,0,10,0xB3,0x55,0x55,0x54,0x55,0x55,0x54,0x44,0x44,0x44,
	0x39,0,2,0x6F,0x08,
	0x39,0,5,0xB4,0x00,0x00,0x00,0x00,
	0x39,0,9,0xB5,0x00,0xE6,0x00,0xB4,0x00,0xD2,0x00,0xA0,
	0x39,0,9,0xB6,0x00,0xAA,0x00,0xC8,0x01,0x28,0x01,0x04,
	0x39,0,7,0xDE,0x10,0x00,0x00,0x00,0x00,0x00,
	0x39,0,27,0xE0,0x80,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x0A,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x1A,
	0x39,0,27,0xE0,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,8,0xB7,0x28,0x28,0x28,0x28,0x28,0x28,0x20,
	0x39,0,2,0x6F,0x11,
	0x39,0,2,0xB7,0x28,
	0x39,0,2,0x6F,0x13,
	0x39,0,2,0xB7,0x20,
	0x39,0,8,0xB8,0x4B,0x4B,0x4B,0x4B,0x4B,0x4B,0x4B,
	0x39,0,2,0x6F,0x0E,
	0x39,0,2,0xB8,0x4B,
	0x39,0,2,0x6F,0x10,
	0x39,0,2,0xB8,0x4B,
	0x39,0,8,0xB9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x1C,
	0x39,0,2,0xB9,0x00,
	0x39,0,2,0x6F,0x20,
	0x39,0,2,0xB9,0x00,
	0x39,0,2,0x6F,0x0E,
	0x39,0,8,0xB9,0x23,0x23,0x23,0x23,0x23,0x23,0x23,
	0x39,0,2,0x6F,0x1E,
	0x39,0,2,0xB9,0x23,
	0x39,0,2,0x6F,0x22,
	0x39,0,2,0xB9,0x23,
	0x39,0,2,0x6F,0x07,
	0x39,0,8,0xB9,0x28,0x25,0x23,0x23,0x23,0x23,0x1E,
	0x39,0,2,0x6F,0x1D,
	0x39,0,2,0xB9,0x23,
	0x39,0,2,0x6F,0x21,
	0x39,0,2,0xB9,0x1E,
	0x39,0,4,0xBA,0x15,0x15,0x15,
	0x39,0,4,0xBB,0x03,0x19,0x19,
	0x39,0,2,0x6F,0x05,
	0x39,0,24,0xBB,0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x27,0x2B,0x2B,0x2E,0x2E,0x31,0x31,0x41,0x41,0x41,0x41,0x41,
	0x39,0,2,0xCD,0x70,
	0x39,0,5,0xD8,0x60,0x60,0x20,0x00,
	0x39,0,2,0x6F,0x06,
	0x39,0,5,0xD8,0x60,0x60,0x20,0x00,
	0x39,0,2,0x6F,0x0C,
	0x39,0,5,0xD8,0x60,0x60,0x20,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,2,0xD8,0x00,
	0x39,0,2,0x6F,0x1D,
	0x39,0,2,0xD8,0x20,
	0x39,0,2,0x6F,0x1E,
	0x39,0,2,0xD8,0x24,
	0x39,0,2,0x6F,0x21,
	0x39,0,4,0xD8,0x56,0x56,0x56,
	0x39,0,2,0x6F,0x24,
	0x39,0,3,0xD8,0x54,0x52,
	0x39,0,2,0x6F,0x0F,
	0x39,0,2,0xB7,0xA0,
	0x39,0,2,0x6F,0x07,
	0x39,0,2,0xE3,0x28,
	0x39,0,2,0x6F,0x04,
	0x39,0,2,0xD9,0x0F,
	0x39,0,2,0x6F,0x06,
	0x39,0,2,0xD9,0x40,
	0x39,0,2,0x6F,0x03,
	0x39,0,3,0xD3,0x00,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x01,
	0x39,0,2,0x6F,0x01,
	0x39,0,5,0xD2,0x01,0x00,0x00,0x20,
	0x39,0,2,0x6F,0x05,
	0x39,0,4,0xD2,0x01,0x01,0x00,
	0x39,0,2,0xDD,0x5F,
	0x39,0,2,0x6F,0x0B,
	0x39,0,3,0xE4,0x00,0x00,
	0x39,0,2,0x6F,0x0D,
	0x39,0,3,0xE4,0x7F,0xFF,
	0x39,0,2,0x6F,0x0E,
	0x39,0,6,0xD2,0x02,0x00,0x00,0x04,0x00,
	0x39,0,2,0x6F,0x02,
	0x39,0,2,0xB0,0x08,
	0x39,0,2,0xE4,0x90,
	0x39,0,2,0x6F,0x0A,
	0x39,0,2,0xE4,0x90,
	0x39,0,2,0x6F,0x18,
	0x39,0,2,0xD8,0x28,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x01,
	0x39,0,2,0x6F,0x05,
	0x39,0,4,0xC5,0x15,0x15,0x15,
	0x39,0,2,0x6F,0x03,
	0x39,0,2,0xC7,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x02,
	0x39,0,2,0xCC,0x30,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x02,
	0x39,0,2,0xCC,0x30,


	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x02,
	0x39,0,2,0xCC,0x30,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x02,
	0x39,0,2,0xBF,0x2B,
	0x39,0,19,0xB0,0x00,0x00,0x01,0xD0,0x02,0x09,0x02,0x80,0x02,0xF9,0x03,0x58,0x03,0xB7,0x04,0x07,0x04,0x7F,
	0x39,0,19,0xB1,0x04,0xEE,0x05,0x3B,0x05,0x88,0x05,0xD5,0x06,0x75,0x06,0xEB,0x07,0x5B,0x07,0xC2,0x08,0x25,
	0x39,0,15,0xB2,0x08,0x87,0x08,0xE6,0x09,0x45,0x09,0xA4,0x09,0xFF,0x0A,0xB2,0x0A,0xB3,
	0x39,0,19,0xB3,0x00,0x00,0x01,0xEC,0x02,0x09,0x02,0x45,0x02,0x8C,0x02,0xD3,0x03,0x1B,0x03,0x5F,0x03,0xCE,
	0x39,0,19,0xB4,0x04,0x37,0x04,0x7E,0x04,0xC5,0x05,0x0C,0x05,0x9B,0x06,0x09,0x06,0x71,0x06,0xD0,0x07,0x2B,
	0x39,0,15,0xB5,0x07,0x7F,0x07,0xCF,0x08,0x1F,0x08,0x6F,0x08,0xC0,0x09,0x68,0x09,0x69,
	0x39,0,19,0xB6,0x00,0x00,0x02,0x7C,0x02,0xB8,0x03,0x35,0x03,0xB4,0x04,0x1B,0x04,0x80,0x04,0xD7,0x05,0x60,
	0x39,0,19,0xB7,0x05,0xDC,0x06,0x35,0x06,0x8E,0x06,0xE7,0x07,0x99,0x08,0x21,0x08,0xA1,0x09,0x17,0x09,0x89,
	0x39,0,15,0xB8,0x09,0xF4,0x0A,0x5C,0x0A,0xC4,0x0B,0x2C,0x0B,0x96,0x0C,0x77,0x0C,0x78,
	0x39,0,2,0xBF,0x2A,
	0x39,0,19,0xB0,0x00,0x00,0x01,0xD0,0x02,0x09,0x02,0x80,0x02,0xF9,0x03,0x58,0x03,0xB7,0x04,0x07,0x04,0x7F,
	0x39,0,19,0xB1,0x04,0xEE,0x05,0x3B,0x05,0x88,0x05,0xD5,0x06,0x75,0x06,0xEB,0x07,0x5B,0x07,0xC2,0x08,0x25,
	0x39,0,15,0xB2,0x08,0x87,0x08,0xE6,0x09,0x45,0x09,0xA4,0x09,0xFF,0x0A,0xB2,0x0A,0xB3,
	0x39,0,19,0xB3,0x00,0x00,0x01,0xEC,0x02,0x09,0x02,0x45,0x02,0x8C,0x02,0xD3,0x03,0x1B,0x03,0x5F,0x03,0xCE,
	0x39,0,19,0xB4,0x04,0x37,0x04,0x7E,0x04,0xC5,0x05,0x0C,0x05,0x9B,0x06,0x09,0x06,0x71,0x06,0xD0,0x07,0x2B,
	0x39,0,15,0xB5,0x07,0x7F,0x07,0xCF,0x08,0x1F,0x08,0x6F,0x08,0xC0,0x09,0x68,0x09,0x69,
	0x39,0,19,0xB6,0x00,0x00,0x02,0x7C,0x02,0xB8,0x03,0x35,0x03,0xB4,0x04,0x1B,0x04,0x80,0x04,0xD7,0x05,0x60,
	0x39,0,19,0xB7,0x05,0xDC,0x06,0x35,0x06,0x8E,0x06,0xE7,0x07,0x99,0x08,0x21,0x08,0xA1,0x09,0x17,0x09,0x89,
	0x39,0,15,0xB8,0x09,0xF4,0x0A,0x5C,0x0A,0xC4,0x0B,0x2C,0x0B,0x96,0x0C,0x77,0x0C,0x78,
	0x39,0,2,0xBF,0x29,
	0x39,0,19,0xB0,0x00,0x00,0x02,0x08,0x02,0x38,0x02,0x72,0x02,0xB7,0x03,0x42,0x03,0xB1,0x04,0x09,0x04,0x63,
	0x39,0,19,0xB1,0x04,0xEB,0x05,0x53,0x05,0xB3,0x06,0x56,0x06,0xEF,0x07,0x70,0x07,0xEC,0x08,0x60,0x08,0xCE,
	0x39,0,15,0xB2,0x09,0xA9,0x0A,0x84,0x0B,0x81,0x0B,0xD9,0x0C,0x05,0x0C,0x57,0x0C,0x6A,
	0x39,0,19,0xB3,0x00,0x00,0x02,0x10,0x02,0x28,0x02,0x43,0x02,0x68,0x02,0xC1,0x03,0x1D,0x03,0x67,0x03,0xB1,
	0x39,0,19,0xB4,0x04,0x2F,0x04,0x91,0x04,0xEB,0x05,0x81,0x06,0x0C,0x06,0x7F,0x06,0xF2,0x07,0x59,0x07,0xB9,
	0x39,0,15,0xB5,0x08,0x78,0x09,0x37,0x0A,0x37,0x0A,0xA9,0x0A,0xED,0x0B,0x53,0x0B,0x66,
	0x39,0,19,0xB6,0x00,0x00,0x02,0xB7,0x02,0xE2,0x03,0x1D,0x03,0x67,0x03,0xFB,0x04,0x7A,0x04,0xDA,0x05,0x35,
	0x39,0,19,0xB7,0x05,0xCF,0x06,0x47,0x06,0xB6,0x07,0x6D,0x08,0x14,0x08,0xA3,0x09,0x32,0x09,0xB9,0x0A,0x3A,
	0x39,0,15,0xB8,0x0B,0x3B,0x0C,0x3C,0x0D,0x82,0x0D,0xFC,0x0E,0x2B,0x0E,0x7E,0x0E,0x91,
	0x39,0,2,0xBF,0x28,
	0x39,0,19,0xB0,0x00,0x00,0x01,0x4B,0x01,0x8E,0x01,0xDC,0x02,0x2B,0x02,0xAA,0x03,0x21,0x03,0x76,0x03,0xCD,
	0x39,0,19,0xB1,0x04,0x4F,0x04,0xC2,0x05,0x1D,0x05,0xBC,0x06,0x49,0x06,0xC7,0x07,0x3D,0x07,0xAC,0x08,0x15,
	0x39,0,15,0xB2,0x08,0xD0,0x09,0x94,0x0A,0x5C,0x0A,0x90,0x0A,0xA9,0x0A,0xC2,0x0A,0xC6,
	0x39,0,19,0xB3,0x00,0x00,0x01,0x3D,0x01,0x6D,0x01,0x92,0x01,0xC6,0x02,0x22,0x02,0x8C,0x02,0xD8,0x03,0x29,
	0x39,0,19,0xB4,0x03,0x9D,0x04,0x0C,0x04,0x63,0x04,0xFA,0x05,0x7A,0x05,0xE8,0x06,0x57,0x06,0xB6,0x07,0x15,
	0x39,0,15,0xB5,0x07,0xC7,0x08,0x6F,0x09,0x1C,0x09,0x49,0x09,0x62,0x09,0x7C,0x09,0x81,
	0x39,0,19,0xB6,0x00,0x00,0x02,0x02,0x02,0x45,0x02,0x8B,0x02,0xDD,0x03,0x61,0x03,0xE7,0x04,0x43,0x04,0x9E,
	0x39,0,19,0xB7,0x05,0x2B,0x05,0xAC,0x06,0x15,0x06,0xCA,0x07,0x66,0x07,0xF0,0x08,0x78,0x08,0xF3,0x09,0x6C,
	0x39,0,15,0xB8,0x0A,0x50,0x0B,0x2E,0x0C,0x1B,0x0C,0x58,0x0C,0x79,0x0C,0x99,0x0C,0x9D,
	0x39,0,2,0xBF,0x27,
	0x39,0,19,0xB0,0x00,0x00,0x01,0x24,0x01,0x44,0x01,0x7D,0x01,0xBC,0x02,0x22,0x02,0x87,0x02,0xD7,0x03,0x15,
	0x39,0,19,0xB1,0x03,0x8E,0x03,0xEE,0x04,0x4E,0x04,0xD1,0x05,0x4F,0x05,0xB8,0x06,0x15,0x06,0x6E,0x06,0xC1,
	0x39,0,15,0xB2,0x07,0x5B,0x07,0xEA,0x08,0x69,0x08,0x88,0x08,0x96,0x08,0xA6,0x08,0xA8,
	0x39,0,19,0xB3,0x00,0x00,0x00,0xF6,0x01,0x29,0x01,0x48,0x01,0x6B,0x01,0xAC,0x01,0xFD,0x02,0x45,0x02,0x7C,
	0x39,0,19,0xB4,0x02,0xEE,0x03,0x45,0x03,0x9C,0x04,0x19,0x04,0x93,0x04,0xF6,0x05,0x4B,0x05,0x9A,0x05,0xE3,
	0x39,0,15,0xB5,0x06,0x71,0x06,0xEE,0x07,0x65,0x07,0x83,0x07,0x91,0x07,0x9E,0x07,0xA1,
	0x39,0,19,0xB6,0x00,0x00,0x01,0x9D,0x02,0x08,0x02,0x33,0x02,0x6C,0x02,0xD4,0x03,0x3C,0x03,0x94,0x03,0xD9,
	0x39,0,19,0xB7,0x04,0x5C,0x04,0xC3,0x05,0x29,0x05,0xBF,0x06,0x4E,0x06,0xC5,0x07,0x2D,0x07,0x8E,0x07,0xE9,
	0x39,0,15,0xB8,0x08,0x99,0x09,0x3A,0x09,0xD2,0x09,0xF9,0x0A,0x0A,0x0A,0x1D,0x0A,0x20,
	0x39,0,2,0xBF,0x26,
	0x39,0,19,0xB0,0x00,0x00,0x00,0xEA,0x01,0x12,0x01,0x3D,0x01,0x5A,0x01,0xBA,0x02,0x12,0x02,0x51,0x02,0x94,
	0x39,0,19,0xB1,0x02,0xF6,0x03,0x4C,0x03,0x91,0x04,0x1E,0x04,0x8F,0x04,0xE9,0x05,0x3E,0x05,0x8B,0x05,0xD1,
	0x39,0,15,0xB2,0x06,0x50,0x06,0xC4,0x07,0x30,0x07,0x47,0x07,0x50,0x07,0x60,0x07,0x62,
	0x39,0,19,0xB3,0x00,0x00,0x00,0xC4,0x00,0xE9,0x01,0x01,0x01,0x14,0x01,0x4F,0x01,0x90,0x01,0xC2,0x01,0xFE,
	0x39,0,19,0xB4,0x02,0x5B,0x02,0xAE,0x02,0xEE,0x03,0x70,0x03,0xDA,0x04,0x30,0x04,0x83,0x04,0xCB,0x05,0x0D,
	0x39,0,15,0xB5,0x05,0x7F,0x05,0xE7,0x06,0x49,0x06,0x5F,0x06,0x6C,0x06,0x75,0x06,0x77,
	0x39,0,19,0xB6,0x00,0x00,0x01,0x40,0x01,0xB4,0x02,0x01,0x02,0x18,0x02,0x6B,0x02,0xC4,0x03,0x04,0x03,0x4B,
	0x39,0,19,0xB7,0x03,0xB7,0x04,0x15,0x04,0x60,0x04,0xF7,0x05,0x73,0x05,0xDA,0x06,0x3B,0x06,0x92,0x06,0xE1,
	0x39,0,15,0xB8,0x07,0x6D,0x07,0xEE,0x08,0x68,0x08,0x83,0x08,0x8E,0x08,0xA0,0x08,0xA2,
	0x39,0,2,0xBF,0x25,
	0x39,0,19,0xB0,0x00,0x00,0x00,0x8E,0x00,0xD9,0x01,0x08,0x01,0x30,0x01,0x99,0x01,0xFE,0x02,0x35,0x02,0x87,
	0x39,0,19,0xB1,0x02,0xE7,0x03,0x44,0x03,0x91,0x04,0x1E,0x04,0x82,0x04,0xE9,0x05,0x35,0x05,0x86,0x05,0xCB,
	0x39,0,15,0xB2,0x06,0x50,0x06,0xC3,0x07,0x2D,0x07,0x46,0x07,0x4E,0x07,0x5F,0x07,0x61,
	0x39,0,19,0xB3,0x00,0x00,0x00,0x66,0x00,0x90,0x00,0xB2,0x00,0xCC,0x01,0x12,0x01,0x66,0x01,0x97,0x01,0xE8,
	0x39,0,19,0xB4,0x02,0x48,0x02,0xA3,0x02,0xEE,0x03,0x70,0x03,0xCD,0x04,0x30,0x04,0x7A,0x04,0xC6,0x05,0x07,
	0x39,0,15,0xB5,0x05,0x7F,0x05,0xE7,0x06,0x47,0x06,0x5E,0x06,0x6A,0x06,0x74,0x06,0x76,
	0x39,0,19,0xB6,0x00,0x00,0x00,0xF6,0x01,0x86,0x01,0xDE,0x01,0xFC,0x02,0x4A,0x02,0xB0,0x02,0xE8,0x03,0x3E,
	0x39,0,19,0xB7,0x03,0xA8,0x04,0x0D,0x04,0x60,0x04,0xF7,0x05,0x65,0x05,0xDA,0x06,0x32,0x06,0x8D,0x06,0xDB,
	0x39,0,15,0xB8,0x07,0x6D,0x07,0xEE,0x08,0x66,0x08,0x82,0x08,0x8D,0x08,0x9F,0x08,0xA1,
	0x39,0,2,0xBF,0x24,
	0x39,0,19,0xB0,0x00,0x00,0x00,0xB2,0x00,0xED,0x01,0x27,0x01,0x61,0x01,0xC3,0x02,0x05,0x02,0x47,0x02,0x89,
	0x39,0,19,0xB1,0x02,0xEB,0x03,0x48,0x03,0x8B,0x04,0x11,0x04,0x79,0x04,0xD8,0x05,0x28,0x05,0x74,0x05,0xB8,
	0x39,0,15,0xB2,0x06,0x29,0x06,0x9E,0x07,0x11,0x07,0x2D,0x07,0x38,0x07,0x41,0x07,0x42,
	0x39,0,19,0xB3,0x00,0x00,0x00,0x9B,0x00,0xBC,0x00,0xDC,0x00,0xFC,0x01,0x38,0x01,0x74,0x01,0xB0,0x01,0xEC,
	0x39,0,19,0xB4,0x02,0x49,0x02,0xA4,0x02,0xE5,0x03,0x68,0x03,0xCD,0x04,0x29,0x04,0x76,0x04,0xBF,0x05,0x00,
	0x39,0,15,0xB5,0x05,0x6A,0x05,0xD4,0x06,0x33,0x06,0x4B,0x06,0x54,0x06,0x60,0x06,0x62,
	0x39,0,19,0xB6,0x00,0x00,0x01,0xD9,0x01,0xFD,0x02,0x20,0x02,0x43,0x02,0x89,0x02,0xCC,0x03,0x0C,0x03,0x4C,
	0x39,0,19,0xB7,0x03,0xAF,0x04,0x12,0x04,0x5B,0x04,0xEE,0x05,0x63,0x05,0xCF,0x06,0x29,0x06,0x7E,0x06,0xCB,
	0x39,0,15,0xB8,0x07,0x4D,0x07,0xCF,0x08,0x46,0x08,0x6A,0x08,0x7D,0x08,0x87,0x08,0x88,
	0x39,0,2,0xBF,0x23,
	0x39,0,19,0xB0,0x00,0x00,0x01,0x44,0x01,0x66,0x01,0x8B,0x01,0xB6,0x01,0xE9,0x02,0x18,0x02,0x51,0x02,0x92,
	0x39,0,19,0xB1,0x02,0xE4,0x03,0x38,0x03,0x78,0x03,0xF7,0x04,0x60,0x04,0xB8,0x05,0x0E,0x05,0x54,0x05,0x95,
	0x39,0,15,0xB2,0x06,0x16,0x06,0x86,0x06,0xE9,0x06,0xFE,0x07,0x11,0x07,0x19,0x07,0x24,
	0x39,0,19,0xB3,0x00,0x00,0x00,0xDA,0x01,0x11,0x01,0x2B,0x01,0x4C,0x01,0x79,0x01,0x94,0x01,0xBA,0x01,0xEA,
	0x39,0,19,0xB4,0x02,0x48,0x02,0x9A,0x02,0xE2,0x03,0x56,0x03,0xB5,0x04,0x0C,0x04,0x61,0x04,0xA1,0x04,0xDA,
	0x39,0,15,0xB5,0x05,0x4E,0x05,0xB5,0x06,0x14,0x06,0x27,0x06,0x32,0x06,0x3A,0x06,0x44,
	0x39,0,19,0xB6,0x00,0x00,0x02,0x3B,0x02,0x66,0x02,0x85,0x02,0xAE,0x02,0xE5,0x03,0x09,0x03,0x2F,0x03,0x60,
	0x39,0,19,0xB7,0x03,0xB8,0x04,0x0E,0x04,0x53,0x04,0xD4,0x05,0x44,0x05,0xA7,0x06,0x07,0x06,0x57,0x06,0x9F,
	0x39,0,15,0xB8,0x07,0x2E,0x07,0xAE,0x08,0x1C,0x08,0x34,0x08,0x41,0x08,0x54,0x08,0x62,
	0x39,0,2,0xBF,0x22,
	0x39,0,19,0xB0,0x00,0x00,0x01,0x80,0x01,0xC8,0x01,0xDD,0x01,0xEC,0x02,0x0B,0x02,0x55,0x02,0x83,0x02,0xA2,
	0x39,0,19,0xB1,0x02,0xF9,0x03,0x3F,0x03,0x7C,0x03,0xF1,0x04,0x56,0x04,0xA7,0x04,0xF7,0x05,0x39,0x05,0x81,
	0x39,0,15,0xB2,0x06,0x01,0x06,0x63,0x06,0xBB,0x06,0xDD,0x06,0xEC,0x06,0xF5,0x06,0xF6,
	0x39,0,19,0xB3,0x00,0x00,0x01,0x0D,0x01,0x4C,0x01,0x6F,0x01,0x86,0x01,0xA5,0x01,0xCD,0x01,0xE6,0x02,0x03,
	0x39,0,19,0xB4,0x02,0x47,0x02,0xA0,0x02,0xDD,0x03,0x53,0x03,0xB3,0x03,0xFE,0x04,0x48,0x04,0x87,0x04,0xC9,
	0x39,0,15,0xB5,0x05,0x3B,0x05,0x97,0x05,0xF5,0x06,0x07,0x06,0x0F,0x06,0x1A,0x06,0x1B,
	0x39,0,19,0xB6,0x00,0x00,0x02,0x6C,0x02,0xD2,0x02,0xEE,0x03,0x10,0x03,0x40,0x03,0x60,0x03,0x7B,0x03,0x97,
	0x39,0,19,0xB7,0x03,0xD5,0x04,0x20,0x04,0x5E,0x04,0xD3,0x05,0x3D,0x05,0x96,0x05,0xED,0x06,0x39,0x06,0x88,
	0x39,0,15,0xB8,0x07,0x16,0x07,0x87,0x07,0xF4,0x08,0x12,0x08,0x22,0x08,0x2C,0x08,0x2D,
	0x39,0,2,0xBF,0x21,
	0x39,0,19,0xB0,0x00,0x00,0x01,0xE4,0x02,0x0D,0x02,0x20,0x02,0x35,0x02,0x58,0x02,0x70,0x02,0xAD,0x02,0xD4,
	0x39,0,19,0xB1,0x03,0x1B,0x03,0x52,0x03,0x84,0x03,0xE6,0x04,0x3F,0x04,0x8A,0x04,0xD5,0x05,0x19,0x05,0x58,
	0x39,0,15,0xB2,0x05,0xCA,0x06,0x38,0x06,0x90,0x06,0xA6,0x06,0xB5,0x06,0xBB,0x06,0xC7,
	0x39,0,19,0xB3,0x00,0x00,0x01,0x68,0x01,0x89,0x01,0xA7,0x01,0xBD,0x01,0xDC,0x01,0xEE,0x02,0x11,0x02,0x27,
	0x39,0,19,0xB4,0x02,0x6B,0x02,0xA0,0x02,0xC7,0x03,0x45,0x03,0xA3,0x03,0xE8,0x04,0x2F,0x04,0x70,0x04,0xA7,
	0x39,0,15,0xB5,0x05,0x10,0x05,0x71,0x05,0xC1,0x05,0xD4,0x05,0xE1,0x05,0xE7,0x05,0xF3,
	0x39,0,19,0xB6,0x00,0x00,0x02,0xEE,0x03,0x28,0x03,0x47,0x03,0x65,0x03,0x95,0x03,0xB2,0x03,0xC7,0x03,0xDF,
	0x39,0,19,0xB7,0x04,0x15,0x04,0x3C,0x04,0x66,0x04,0xD4,0x05,0x2E,0x05,0x7D,0x05,0xCD,0x06,0x19,0x06,0x5B,
	0x39,0,15,0xB8,0x06,0xDC,0x07,0x55,0x07,0xB8,0x07,0xD2,0x07,0xE1,0x07,0xE9,0x07,0xF9,
	0x39,0,2,0xBF,0x20,
	0x39,0,19,0xB0,0x00,0x00,0x02,0x2F,0x02,0x4F,0x02,0x6B,0x02,0x71,0x02,0x94,0x02,0xB5,0x02,0xD9,0x02,0xEC,
	0x39,0,19,0xB1,0x03,0x0E,0x03,0x3C,0x03,0x6D,0x03,0xCC,0x04,0x08,0x04,0x3B,0x04,0x7E,0x04,0xB3,0x04,0xE6,
	0x39,0,15,0xB2,0x05,0x51,0x05,0xA8,0x05,0xF2,0x06,0x15,0x06,0x1C,0x06,0x21,0x06,0x25,
	0x39,0,19,0xB3,0x00,0x00,0x01,0xA1,0x01,0xBA,0x01,0xD2,0x01,0xE8,0x02,0x04,0x02,0x1A,0x02,0x2C,0x02,0x44,
	0x39,0,19,0xB4,0x02,0x5E,0x02,0x85,0x02,0xB2,0x03,0x0F,0x03,0x4F,0x03,0x92,0x03,0xDA,0x04,0x0D,0x04,0x41,
	0x39,0,15,0xB5,0x04,0xA4,0x04,0xF6,0x05,0x41,0x05,0x4F,0x05,0x56,0x05,0x61,0x05,0x65,
	0x39,0,19,0xB6,0x00,0x00,0x03,0x3F,0x03,0x6D,0x03,0x93,0x03,0xA5,0x03,0xCE,0x03,0xF1,0x04,0x08,0x04,0x17,
	0x39,0,19,0xB7,0x04,0x37,0x04,0x55,0x04,0x76,0x04,0xBE,0x04,0xF3,0x05,0x33,0x05,0x79,0x05,0xAF,0x05,0xE6,
	0x39,0,15,0xB8,0x06,0x59,0x06,0xB9,0x07,0x14,0x07,0x2D,0x07,0x39,0x07,0x3F,0x07,0x44,
	0x39,0,2,0xCE,0x01,
	0x39,0,2,0xCC,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x02,
	0x39,0,25,0xB9,0x00,0x04,0x00,0x0C,0x00,0x14,0x00,0x1C,0x00,0x2C,0x00,0x3C,0x00,0x4C,0x00,0x5C,0x00,0x7C,0x00,0x9C,0x00,0xBC,0x00,0xFC,
	0x39,0,25,0xBA,0x01,0x3C,0x01,0x7C,0x01,0xBC,0x01,0xFC,0x02,0x3C,0x02,0xBC,0x03,0x3C,0x03,0xBC,0x03,0xDC,0x03,0xEC,0x03,0xFC,0x03,0xFF,


	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x03,
	0x39,0,11,0xB0,0x3F,0x00,0x3F,0x3F,0x3F,0x3F,0x00,0x3F,0x3F,0x3F,
	0x39,0,2,0x6F,0x0B,
	0x39,0,11,0xB0,0x3F,0x00,0x3F,0x3F,0x3F,0x3F,0x00,0x3F,0x3F,0x3F,
	0x39,0,2,0x6F,0x15,
	0x39,0,11,0xB0,0x3F,0x00,0x3F,0x3F,0x3F,0x3F,0x00,0x3F,0x3F,0x3F,
	0x39,0,2,0x6F,0x29,
	0x39,0,11,0xB0,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
	0x39,0,2,0x6F,0x33,
	0x39,0,11,0xB0,0x3F,0x00,0x3F,0x3F,0x3F,0x3F,0x00,0x3F,0x3F,0x3F,
	0x39,0,2,0x6F,0x3D,
	0x39,0,11,0xB0,0x3F,0x00,0x3F,0x3F,0x3F,0x3F,0x00,0x3F,0x3F,0x3F,
	0x39,0,2,0x6F,0x47,
	0x39,0,11,0xB0,0x3F,0x00,0x3F,0x3F,0x3F,0x3F,0x00,0x3F,0x3F,0x3F,
	0x39,0,2,0x6F,0x5B,
	0x39,0,11,0xB0,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,
	0x39,0,2,0xB2,0x02,
	0x39,0,2,0x6F,0x01,
	0x39,0,4,0xB2,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x04,
	0x39,0,5,0xB2,0x05,0x09,0x05,0x0A,
	0x39,0,2,0x6F,0x08,
	0x39,0,4,0xB2,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x0B,
	0x39,0,4,0xB2,0x00,0x02,0x02,
	0x39,0,2,0x6F,0x11,
	0x39,0,4,0xB2,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x14,
	0x39,0,5,0xB2,0x05,0x09,0x05,0x0A,
	0x39,0,2,0x6F,0x18,
	0x39,0,5,0xB2,0x60,0x0A,0x60,0x0B,
	0x39,0,2,0x6F,0x20,
	0x39,0,5,0xB2,0x50,0x06,0x50,0x05,
	0x39,0,2,0x6F,0x24,
	0x39,0,3,0xB2,0x00,0x00,
	0x39,0,2,0x6F,0x2B,
	0x39,0,3,0xB2,0x01,0x01,
	0x39,0,2,0xB6,0x00,
	0x39,0,2,0x6F,0x01,
	0x39,0,4,0xB6,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x04,
	0x39,0,4,0xB6,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x07,
	0x39,0,4,0xB6,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x0D,
	0x39,0,4,0xB6,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x11,
	0x39,0,3,0xB6,0x00,0x08,
	0x39,0,2,0x6F,0x1A,
	0x39,0,3,0xB6,0x00,0x08,
	0x39,0,2,0x6F,0x23,
	0x39,0,3,0xB6,0x00,0x08,
	0x39,0,2,0x6F,0x35,
	0x39,0,3,0xB6,0x00,0x08,
	0x39,0,6,0xBB,0x22,0x10,0x00,0x00,0x48,
	0x39,0,2,0x6F,0x05,
	0x39,0,4,0xBB,0x00,0x02,0x48,
	0x39,0,2,0x6F,0x08,
	0x39,0,4,0xBB,0x00,0x02,0x48,
	0x39,0,2,0x6F,0x0E,
	0x39,0,4,0xBB,0x00,0x00,0x24,
	0x39,0,6,0xBC,0x11,0x00,0x00,0x12,0x58,
	0x39,0,2,0x6F,0x05,
	0x39,0,4,0xBC,0x00,0x12,0x58,
	0x39,0,2,0x6F,0x08,
	0x39,0,4,0xBC,0x00,0x74,0xB0,
	0x39,0,2,0x6F,0x0E,
	0x39,0,4,0xBC,0x00,0x12,0x58,
	0x39,0,4,0xC3,0x00,0x02,0x22,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x03,
	0x39,0,2,0xC7,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x03,
	0x39,0,2,0x6F,0x15,
	0x39,0,3,0xC6,0x1B,0x03,
	0x39,0,2,0x6F,0x17,
	0x39,0,3,0xC6,0xA0,0x0A,
	0x39,0,2,0x6F,0x19,
	0x39,0,5,0xC6,0x55,0x55,0x50,0x15,
	0x39,0,2,0x6F,0x1D,
	0x39,0,5,0xC6,0x00,0x00,0x00,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0x6F,0x2E,
	0x39,0,6,0xC0,0x23,0x02,0x30,0x00,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x04,
	0x39,0,2,0xC2,0x14,
	0x39,0,2,0xB5,0x00,
	0x39,0,2,0x6F,0x01,
	0x39,0,4,0xB5,0x00,0x14,0x01,
	0x39,0,2,0x6F,0x04,
	0x39,0,2,0xB5,0xE4,
	0x39,0,2,0x6F,0x0C,
	0x39,0,4,0xB6,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x0F,
	0x39,0,4,0xB6,0x0E,0x10,0x00,
	0x39,0,2,0x6F,0x18,
	0x39,0,4,0xB6,0x0E,0x11,0x00,
	0x39,0,2,0x6F,0x1B,
	0x39,0,4,0xB6,0x05,0xB6,0x80,
	0x39,0,2,0x6F,0x00,
	0x39,0,4,0xB8,0x13,0xC8,0x00,
	0x39,0,2,0x6F,0x03,
	0x39,0,3,0xB8,0x06,0xC6,  // 20231104
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x05,
	0x39,0,4,0xB0,0x03,0x21,0x01,
	0x39,0,3,0xB3,0x86,0x85,
	0x39,0,3,0xB5,0x86,0x85,
	0x39,0,5,0xB7,0x85,0x00,0x00,0x85,
	0x39,0,5,0xB8,0x85,0x00,0x00,0x85,
	0x39,0,3,0xC7,0x03,0x01,
	0x39,0,5,0xE0,0x02,0x00,0x00,0x00,
	0x39,0,4,0xD1,0x00,0x0A,0x10,
	0x39,0,5,0xE1,0x00,0x09,0x80,0x00,
	0x39,0,2,0xCA,0x13,
	0x39,0,3,0xCB,0x03,0X03,
	0x39,0,2,0xD7,0x13,
	0x39,0,2,0xD8,0x13,
	0x39,0,2,0xD9,0x40,
	0x39,0,2,0x6F,0x06,
	0x39,0,2,0xEC,0x40,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x06,
	0x39,0,6,0xB0,0x08,0x38,0x14,0x38,0x15,
	0x39,0,6,0xB1,0x38,0x18,0x38,0x19,0x38,
	0x39,0,6,0xB2,0x38,0x00,0x38,0x38,0x38,
	0x39,0,6,0xB3,0x38,0x37,0x37,0x37,0x37,
	0x39,0,6,0xB6,0x08,0x38,0x14,0x38,0x15,
	0x39,0,6,0xB7,0x38,0x18,0x38,0x19,0x38,
	0x39,0,6,0xB8,0x38,0x00,0x38,0x38,0x38,
	0x39,0,6,0xB9,0x38,0x37,0x37,0x37,0x37,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x07,
	0x39,0,6,0xB0,0x84,0xC0,0x78,0x70,0x00,   // 20231104
	0x39,0,7,0xB1,0x08,0x18,0x00,0x18,0x08,0x00,
	0x39,0,10,0xB2,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x39,0,2,0x6F,0x36,
	0x39,0,10,0xB2,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,
	0x39,0,2,0x6F,0x3F,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,2,0x6F,0x09,
	0x39,0,10,0xB2,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x39,0,2,0x6F,0x48,
	0x39,0,10,0xB2,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,
	0x39,0,2,0x6F,0x51,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,2,0x6F,0x12,
	0x39,0,10,0xB2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x5A,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,2,0x6F,0x63,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,2,0x6F,0x1B,
	0x39,0,10,0xB2,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x39,0,2,0x6F,0x6C,
	0x39,0,10,0xB2,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,
	0x39,0,2,0x6F,0x75,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,2,0x6F,0x24,
	0x39,0,10,0xB2,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,0x90,
	0x39,0,2,0x6F,0x7E,
	0x39,0,10,0xB2,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,0x56,
	0x39,0,2,0x6F,0x87,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,2,0x6F,0x2D,
	0x39,0,10,0xB2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x90,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,2,0x6F,0x99,
	0x39,0,10,0xB2,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,0x04,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x07,
	0x39,0,14,0xB4,0xC0,0x60,0x80,0x80,0x60,0x40,0x60,0x80,0x80,0x80,0x80,0x40,0x80,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x08,
	0x39,0,3,0xC1,0x8F,0x7F,
	//0x39,0,3,0xC1,0x0F,0xFF,
	0x39,0,5,0xFF,0xAA,0x55,0xA5,0x81,
	0x39,0,2,0x6F,0x19,
	0x39,0,2,0xFB,0x30,
	0x39,0,2,0x6F,0x05,
	0x39,0,2,0xFE,0x34,
	0x39,0,5,0xFF,0xAA,0x55,0xA5,0x80,
	0x39,0,2,0x6F,0x1A,
	0x39,0,2,0xF4,0x55,
	0x39,0,5,0xFF,0xAA,0x55,0xA5,0x83,
	0x39,0,2,0x6F,0x12,
	0x39,0,2,0xFE,0x41,

	#if 1
	/* Round */
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x07,
	0x39,0,4,0xC0,0x87,0x01,0x08,
	0x39,0,2,0x6F,0x03,
	0x39,0,15,0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC1,0x36,0x00,0x01,0x00,0x02,0xD9,0x03,0x60,0x00,0x00,0x00,0x00,0x3F,0xFD,0xD7,0xC0,0x80,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC1,0x0F,0x34,0xD4,0x18,0x00,0x00,0x10,0x34,0x30,0x10,0xBD,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC1,0x44,0x25,0x2F,0x00,0x14,0x1C,0x00,0x00,0x00,0x0F,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC2,0x36,0x00,0x00,0x04,0x00,0x04,0x00,0x08,0x00,0x00,0x01,0x40,0x3F,0xFF,0xFE,0x20,0x00,0x00,0x00,0x01,0xE4,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC2,0x04,0x44,0x19,0x24,0x00,0x11,0x1C,0x44,0x30,0x0A,0xF6,0x00,0x00,0xC0,0x00,0xFF,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC2,0x44,0x19,0x1D,0x00,0x05,0x10,0x00,0x00,0x00,0x04,0x1F,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC3,0x3E,0x00,0x00,0xA9,0x01,0x00,0x01,0xA0,0x00,0x00,0xE2,0xE0,0x3F,0xFF,0x5F,0x4C,0x00,0x00,0x00,0xA1,0xB4,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC3,0x3D,0x44,0x25,0x37,0x00,0x1D,0x5B,0x43,0x30,0x3D,0xEE,0x00,0x00,0xC0,0x00,0xFF,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC3,0x00,0x1A,0x1E,0x00,0x05,0x10,0x00,0x00,0x00,0x3D,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC4,0x30,0x0F,0x01,0x00,0x02,0xD9,0xFC,0xA0,0x3F,0xFF,0x7A,0x00,0x3F,0xFE,0xB9,0xE0,0x80,0x00,0x12,0x10,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC4,0x0F,0x00,0x1F,0x63,0x00,0x00,0x10,0x34,0xF0,0xF0,0xBD,0x04,0x30,0x30,0xFF,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC4,0x00,0x08,0x12,0x00,0x14,0x1C,0x00,0x00,0x00,0x0F,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC5,0x30,0x0F,0x00,0x04,0x00,0x04,0xFF,0xF8,0x3F,0xFF,0xFE,0x70,0x3F,0xFF,0xFE,0x70,0x00,0x00,0x00,0x11,0x30,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC5,0x04,0x00,0x13,0x1E,0x00,0x11,0x1C,0x44,0xF0,0xF6,0xF6,0x00,0x64,0xF0,0xFF,0xFF,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC5,0x44,0x19,0x1D,0x99,0x4F,0x5A,0x00,0x00,0x00,0x04,0x1F,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC6,0x38,0x0F,0x00,0xA9,0x01,0x00,0xFE,0x60,0x3F,0xFF,0x05,0x5C,0x3F,0xFF,0x7C,0x8C,0x80,0x00,0x11,0x4E,0x18,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC6,0x3D,0x00,0x00,0x12,0x00,0x1D,0x5B,0x43,0xF0,0xC3,0xEE,0x04,0x4A,0xC0,0x00,0xFF,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC6,0x44,0x25,0x2F,0x99,0x43,0x4B,0x00,0x00,0x00,0x3D,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC7,0x3A,0x0F,0x01,0x00,0x02,0xD9,0xFC,0xA0,0x00,0x00,0x36,0x00,0x00,0x01,0xCD,0x20,0x7F,0xFF,0xE0,0x55,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC7,0x00,0x34,0xD4,0x18,0x99,0x4F,0x5F,0x34,0x0F,0x10,0x43,0xFB,0xD0,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC7,0x00,0x1A,0x1E,0x99,0x4F,0x5A,0x00,0x00,0x00,0x00,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC8,0x3A,0x0F,0x00,0x04,0x00,0x04,0xFF,0xF8,0x00,0x00,0x01,0x90,0x00,0x00,0x01,0x90,0xFF,0xFF,0xFF,0xEE,0xD0,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC8,0x05,0x44,0x19,0x24,0x99,0x43,0x4E,0x44,0x0F,0x0A,0x0A,0xFF,0x9C,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC8,0x00,0x08,0x12,0x99,0x43,0x4B,0x00,0x00,0x00,0x05,0x1F,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xC9,0x2A,0x0F,0x00,0xA9,0x01,0x00,0xFE,0x60,0x00,0x01,0x46,0x00,0x00,0x00,0x26,0xB4,0xFF,0xFF,0xE8,0x3E,0x1C,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xC9,0x3D,0x44,0x25,0x37,0x99,0x04,0x42,0x43,0x0F,0x3D,0x12,0xFB,0xB6,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xC9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3D,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xCA,0x2C,0x00,0x01,0x00,0x02,0xD9,0x03,0x60,0x3F,0xFF,0x44,0x00,0x00,0x00,0xEB,0x00,0x00,0x00,0x00,0xBD,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xCA,0x00,0x00,0x1F,0x63,0x99,0x4F,0x5F,0x34,0xC0,0xF0,0x43,0x00,0x00,0x30,0xFF,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xCA,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xCB,0x2C,0x00,0x00,0x04,0x00,0x04,0x00,0x08,0x3F,0xFF,0xFE,0x20,0x00,0x00,0x01,0x40,0x00,0x00,0x00,0x01,0xE4,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xCB,0x05,0x00,0x13,0x1E,0x99,0x43,0x4E,0x44,0xC0,0xF6,0x0A,0x00,0x00,0x30,0xFF,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xCB,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0x1F,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xCC,0x2C,0x00,0x00,0xA9,0x01,0x00,0x01,0xA0,0x3F,0xFE,0xA2,0x3C,0x00,0x00,0x09,0x74,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xCC,0x3D,0x00,0x00,0x12,0x99,0x04,0x42,0x43,0xC0,0xC3,0x12,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xCC,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3D,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xCD,0x21,0x00,0x03,0x49,0x03,0x84,0x00,0x00,0x3F,0xFF,0x3A,0xE4,0x3F,0xFF,0x34,0x18,0x00,0x00,0x0D,0x24,0x75,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xCD,0x08,0x12,0xFB,0x1A,0x00,0x17,0x35,0x55,0xF0,0xE3,0xE2,0x03,0x66,0xF0,0xFF,0xFF,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xCD,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xCE,0x2D,0x00,0x04,0x00,0x03,0x84,0x00,0x00,0x3F,0xFF,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xCE,0x17,0x12,0xFB,0x1A,0x00,0x36,0x57,0x55,0xC0,0xE0,0x1E,0x00,0x00,0x30,0xFF,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xCE,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x17,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xCF,0x2B,0x00,0x04,0x00,0x03,0xC1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7F,0xFF,0xF0,0xFC,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xCF,0x16,0x22,0x1B,0x3B,0x00,0x36,0x57,0x55,0x0F,0x20,0x1F,0xFC,0x20,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xCF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x16,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD0,0x27,0x00,0x03,0x49,0x03,0xC1,0x00,0x00,0x00,0x00,0x00,0x00,0x3F,0xFF,0x26,0x46,0x00,0x00,0x00,0xDD,0x7B,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD0,0x09,0x22,0x1B,0x3B,0x00,0x17,0x35,0x55,0x30,0x1D,0xE1,0x00,0x00,0xC0,0x00,0xFF,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x09,0x40,0x40,0x40,0x40,0x40,0x40,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD3,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD4,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD5,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xD9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xD9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xD9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x00,
	0x39,0,22,0xEA,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x15,
	0x39,0,17,0xEA,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x25,
	0x39,0,17,0xEA,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0x6F,0x13,
	0x39,0,25,0xDF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x39,0,10,0x88,0x01,0x02,0x1C,0x08,0x68,0x00,0x00,0x00,0x00,
	/* Round */
	#endif

	0x39,0,2,0x5F,0x01,
	0x39,0,2,0x26,0x00,
	0x39,0,2,0x17,0x01,  // 10 video mode 01 command mode
	0x39,0,5,0x2A,0x00,0x00,0x04,0x37,
	0x39,0,5,0x2B,0x00,0x00,0x09,0x5F,
	0x39,0,2,0x2F,0x02,
	0x39,0,2,0x35,0x30,
	0x39,0,3,0x44,0x00,0x00,
	0x39,0,3,0x51,0x0E,0x50,
	//0x39,0,3,0x51,0x0D,0xBB,
	0x39,0,2,0x6F,0x04,
	0x39,0,3,0x51,0x0F,0xFE,
	0x39,0,2,0x53,0x20,
	0x39,0,3,0x90,0x00,0x00,
	0x39,0,19,0x91,0x89,0x28,0x00,0x0C,0xD2,0x00,0x02,0x25,0x01,0x14,0x00,0x07,0x09,0x75,0x08,0x7A,0x10,0xF0,
	0x39,0,5,0xFF,0xAA,0x55,0xA5,0x81,
	0x39,0,2,0x6F,0x0E,
	0x39,0,3,0xF5,0x2B,0x00,
	0x39,0,2,0x6F,0x02,
	0x39,0,2,0xF9,0x04,
	0x39,0,5,0xFF,0xAA,0x55,0xA5,0x80,
	0x39,0,2,0x6F,0x0F,
	0x39,0,2,0xFC,0x00,
	0x39,0,2,0x6F,0x09,
	0x39,0,3,0xFC,0xFC,0xF0,
	0x39,0,5,0xFF,0xAA,0x55,0xA5,0x80,
	0x39,0,2,0x6F,0x15,
	0x39,0,3,0xF8,0x01,0x4E,
	0x39,0,2,0x6F,0x31,
	0x39,0,3,0xF8,0x00,0xFD,
	0x39,0,2,0x6F,0x01,
	0x39,0,2,0x1F,0x06,
	0x39,0,2,0x6F,0x19,
	0x39,0,2,0xF2,0x00,
	0x39,0,2,0x6F,0x2D,
	0x39,0,2,0xFC,0x44,
	0x39,0,2,0x6F,0x0A,
	0x39,0,4,0xF6,0x70,0x70,0x70,
	0x39,0,2,0x6F,0x0E,
	0x39,0,2,0xF6,0x70,
	0x39,0,2,0x6F,0x2D,
	0x39,0,2,0xFC,0x44,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00  ,
	0x39,0,2,0x6F,0x06,
	0x39,0,5,0xB5,0x7F,0x4B,0x29,0x4F,
	0x39,0,2,0x6F,0x0C,
	0x39,0,4,0xB5,0x4E,0x25,0x50,
	0x39,0,2,0x6F,0x11,
	0x39,0,6,0xB5,0x29,0x29,0x29,0x29,0x29,
	0x39,0,2,0x6F,0x18,
	0x39,0,6,0xB5,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x1D,
	0x39,0,6,0xB5,0x00,0x00,0x00,0x00,0x00,
	0x39,0,2,0x6F,0x2C,
	0x39,0,2,0xB5,0x00,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0x6F,0xD9,
	0x39,0,3,0xBA,0x00,0x20,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0x6F,0xD1,
	0x39,0,3,0xBA,0x00,0x0A,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	0x39,0,2,0x6F,0x1A,
	0x39,0,2,0xC2,0x1B,
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x04,
	0x39,0,2,0xB5,0x00,
	
	
	0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00, 
	0x39,0,3,0x51,0x00,0x00,

	// bist mode 
	//0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00,
	//0x39,0,8,0xEF,0x00,0x00,0xFF,0xFF,0xFF,0x36,0xEC,
	//0x39,0,2,0xEE,0x01,

	///BIST 
	// 0x39,0,6,0xF0,0x55,0xAA,0x52,0x08,0x00, 
	// 0x39,0,6,0xEF,0x01,0x00,0xFF,0xFF,0xFF, 
	// 0x39,0,2,0xEE,0x01,
	 
};

#endif

/* Soft swire */

static void soft_swire_start(uint8_t pulse)
{
	do
	{
		hal_gpio_init_output( SWIRE_EN, IO_LVL_LOW  ); // 3us
		delayUs(10);
		hal_gpio_init_output( SWIRE_EN, IO_LVL_HIGH ); // 3us
		delayUs(10);
	}while(pulse--);
}

static void soft_swire_timer_callback(void *data)
{
	s_swire_pwm_counter++;
	if( s_swire_pwm_counter < SWIRE_MAX_NUM )
	{
		soft_swire_start(SWIRE_DEFAULT_NUM);
		hal_timer_start(SOFT_SWIRE_TIMER, 16, soft_swire_timer_callback, NULL);
	}
	else
	{
		hal_timer_deinit(SOFT_SWIRE_TIMER);
		hal_timer_stop(SOFT_SWIRE_TIMER);	
		hal_gpio_init_output(SWIRE_EN, IO_LVL_HIGH);
	}
}

static void soft_swire_init(void)
{
	hal_gpio_init_output(SWIRE_EN, IO_LVL_HIGH);
	delayMs(3);
	s_swire_pwm_counter = 0;
	soft_swire_start(SWIRE_DEFAULT_NUM);
	hal_timer_init(SOFT_SWIRE_TIMER);
    hal_timer_start(SOFT_SWIRE_TIMER, 16, soft_swire_timer_callback, NULL);
}

/**
* @brief  panel init
* @param  none
* @retval none
*/
static void app_init_panel(void)
{
    /* reset panel*/
    app_tx_panel_reset();
		
#if LCD_NT36525B
    hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x11);
    delayMs(120);    //120ms
    hal_dsi_tx_ctrl_write_cmd(0x05, 0, 1, 0x29);
    delayMs(20);    //20ms

#elif OLED_NT37705_TM678
    send_panel_init_code( sizeof( panel_init_code ), panel_init_code );
	
//	//	if(sg_aod_in_flag)	// AOD时 51发送比较早，后续不发送造成AOD不亮
//		{
//			sg_aod_in_flag = false;
//	//		hal_dsi_tx_ctrl_write_cmd(0x39, 0, 3, 0x51, 0x03, 0x40);
//		}
    hal_dsi_tx_ctrl_write_cmd( 0x05, 0, 1, 0x11 );
    hal_gpio_init_output( AVDD_EN, IO_LVL_HIGH ); // AVDD_EN
    delayMs( 120 );                         
//    soft_swire_init();
    //delayMs( 30 ); 
#if AP_SWIRE_OUTPUT
    hal_swire_enable(true);
    hal_swire_set_pulse(SWIRE_DEFAULT_PULSE);
#endif
	//	delayMs( 60 ); 
#endif
}

#if TX_CMD_MODE_WITHOUT_TE
/**
* @brief  MIPI RX事件处理函数,demo code 用于command mode 输出发送数据（模式1）
* @param  event:RX事件
* @param  data: user data
* @retval none
*/
static void app_rx_event_cb(hal_rx_event_e event, void *data)
{
    if (panel_display_done && event == HAL_RX_LINE_EVENT)
    {
        hal_dsi_tx_ctrl_gen_a_frame();
    }
}
#endif

#if TX_USE_CMD_MODE
/**
* @brief  TE引脚作为GPIO 输入回调函数,demo code 用于command mode 输出发送数据（模式2）
* @param  none
* @retval none
*/
static void app_tx_cmd_panel_te_cb(void *data)
{
    if (panel_display_done)
    {
        //delayUs(25); /* 撕裂调试 */
        hal_dsi_tx_ctrl_gen_a_frame();
    }
}

/**
* @brief  注册屏端TE信号输入引脚回调函数
* @param  pad :TE输入pad
* @param  trig:触发中断沿配置
* @retval none
*/
static void app_tx_cmd_app_init_panel_te_int(io_pad_e pad, sys_cfg_trigger_e trig)
{
    /*1.关闭中断*/
    hal_gpio_ctrl_eint(pad, DISABLE);

    /*2.中断初始化*/
    hal_gpio_init_eint(pad, trig);

    /*3.注册回调*/
    hal_gpio_reg_eint_cb(pad, app_tx_cmd_panel_te_cb);

    /*4.使能中断*/
    hal_gpio_ctrl_eint(pad, ENABLE);
}
#endif

uint8_t t_pps[128] = {
	0x11,0x00,0x00,0xAB,0x30,0x80,0x0C,0x80,
	0x05,0xA0,0x00,0x19,0x02,0xD0,0x02,0xD0,
	0x02,0x00,0x02,0xC2,0x00,0x20,0x02,0x67,
	0x00,0x0A,0x00,0x0F,0x05,0x00,0x03,0x0D,
	0x18,0x00,0x10,0xF0,0x07,0x10,0x20,0x00,
	0x06,0x0F,0x0F,0x33,0x0E,0x1C,0x2A,0x38,
	0x46,0x54,0x62,0x69,0x70,0x77,0x79,0x7B,
	0x7D,0x7E,0x02,0x02,0x22,0x00,0x2A,0x40,
	0x2A,0xBE,0x3A,0xFC,0x3A,0xFA,0x3A,0xF8,
	0x3B,0x38,0x3B,0x78,0x3B,0xB6,0x4B,0xB6,
	0x4B,0xF4,0x4B,0xF4,0x6C,0x34,0x84,0x74,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

/**
* @brief  mipi rx 初始化
* @param  none
* @retval none
*/
static void app_mipi_rx_init(void)
{
    if (g_rx_ctrl_handle == NULL)
    {
        /* 创建rx   ctrl handle */
        g_rx_ctrl_handle = hal_dsi_rx_ctrl_create_handle();
    }
    /* 配置参数 */
    g_rx_ctrl_handle->base_info.src_w = INPUT_WIDTH;
    g_rx_ctrl_handle->base_info.src_h = INPUT_HEIGHT;
    g_rx_ctrl_handle->base_info.dst_w = OUTPUT_WIDTH;
    g_rx_ctrl_handle->base_info.dst_h = OUTPUT_HEIGHT;
    g_rx_ctrl_handle->base_info.src_frate = INPUT_FRAME_RATE;
    g_rx_ctrl_handle->base_info.src_mode = INPUT_DATA_MODE;
    g_rx_ctrl_handle->base_info.dst_mode = OUTPUT_DATA_MODE;
    g_rx_ctrl_handle->rx_color_mode = INPUT_COLOR_MODE;
    g_rx_ctrl_handle->rx_lanes = INPUT_MIPI_LANE_NUM;
    g_rx_ctrl_handle->rx_nonburst_models = INPUT_VIDEO_MODEL; /* 可不配置 */
    g_rx_ctrl_handle->compress_en = INPUT_COMPRESS;
    g_rx_ctrl_handle->rx_hsclk_rate = INPUT_MIPI_LANE_RATE;
    g_rx_ctrl_handle->cus_dcs_entry_table = g_cus_rx_dcs_execute_table; /* 注册 DCS处理列表 */
    //用的是硬件回复，取消软件回复
    //g_rx_ctrl_handle->rx_dcs_read_entry = ap_dcs_read; /* 注册dsc read 回调函数,可选,此函数为空时由cus_dcs_entry_table执行 */
#if RX_RESOLUTION_CHANGE_ENABLE
    g_rx_ctrl_handle->pps_update_entry = pps_update_handle;
#endif

    //必须打开，否则切换到高分辨率花屏
    g_rx_ctrl_handle->hight_performan_mode = HIGHT_PERFORMAN_L2;
    g_rx_ctrl_handle->err_handler_level = ERR_HANDLE_L2; 


    /* 提前预置PPS,  AP 有PPS cmd也会更新 */
    if (g_rx_ctrl_handle->compress_en == true)
    {
        hal_dsi_rx_ctrl_pre_init_pps(g_rx_ctrl_handle, t_pps, 128);
    }

    /* 初始化rx ctrl */
    hal_dsi_rx_ctrl_init(g_rx_ctrl_handle);

#if RX_READ_HW_ACK
    /* 配置硬件回复 */
    app_set_dcs_hw_ack();
#endif
    hal_dsi_rx_ctrl_start(g_rx_ctrl_handle);


    //hal_dsi_rx_ctrl_set_check_crc(g_rx_ctrl_handle, false);
    
#if TX_CMD_MODE_WITHOUT_TE
    /* 注册接收一帧帧头事件回调,每接收一帧数据TX再往外发一帧 */
    //hal_dsi_rx_ctrl_register_callback(g_rx_ctrl_handle, app_rx_event_cb, HAL_RX_FS_EVENT, true, NULL);
    /* 注册接收第0行数据事件,接收到数据后再往外发送数据,确保不撕裂 */
    uint32_t line = 0;
    hal_dsi_rx_ctrl_register_callback(g_rx_ctrl_handle, app_rx_event_cb, HAL_RX_LINE_EVENT, true, &line);
#endif

#if RX_START_WITHOUT_RST
    /* 等待ap reset置位再启动rx,否则容易收到错误数据 */
    //hal_dsi_rx_ctrl_start(g_rx_ctrl_handle);
#else
//    /* 注册RX start callback,确认RX LP11时再启动RX,防止接收错误数据 */
//    hal_gpio_set_ap_reset_int(ENABLE, app_mipi_rx_start_cb, DETECT_HIGH_LVL);
#endif
}

/**
* @brief  mipi tx 初始化
* @param  none
* @retval none
*/
static void app_mipi_tx_init(void)
{
    if (g_tx_ctrl_handle == NULL)
    {
        g_tx_ctrl_handle = hal_dsi_tx_ctrl_create_handle();
    }
    g_tx_ctrl_handle->channel_id = OUTPUT_VC;
    g_tx_ctrl_handle->lane_num = OUTPUT_LANE_NUMBER;
    g_tx_ctrl_handle->vid_mode = OUTPUT_VIDEO_MODEL;
    g_tx_ctrl_handle->cmd_tx_type = TX_INIT_TYPE;
    g_tx_ctrl_handle->dpi_vsa = OUTPUT_VSA;
    g_tx_ctrl_handle->dpi_vbp = OUTPUT_VBP;
    g_tx_ctrl_handle->dpi_vfp = OUTPUT_VFP;
    g_tx_ctrl_handle->dpi_hsa = OUTPUT_HSA;
    g_tx_ctrl_handle->dpi_hbp = OUTPUT_HBP;
    g_tx_ctrl_handle->dpi_hfp = OUTPUT_HFP;
    g_tx_ctrl_handle->base_info.src_w = INPUT_WIDTH;
    g_tx_ctrl_handle->base_info.src_h = INPUT_HEIGHT;
    g_tx_ctrl_handle->base_info.dst_w = OUTPUT_WIDTH;
    g_tx_ctrl_handle->base_info.dst_h = OUTPUT_HEIGHT;
    g_tx_ctrl_handle->base_info.src_frate = INPUT_FRAME_RATE;
    g_tx_ctrl_handle->base_info.src_mode = INPUT_DATA_MODE;
    g_tx_ctrl_handle->base_info.dst_mode = OUTPUT_DATA_MODE;

    hal_dsi_tx_ctrl_init(g_tx_ctrl_handle);

    /* FIXME set tear on*/
    //要去掉，否则开机启动过程花屏
    //hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, TE_60HZ_MODE);

    /* AP 没有发送数据时默认的显示颜色, 量产为0 0 0(黑色), 配置其他颜色仅为debug使用 */
    hal_dsi_tx_ctrl_set_overwrite_rgb(0x00, 0x00, 0x00);
}

#if !RX_START_WITHOUT_RST
/**
* @brief  mipi rx start函数,开启AP RSTN等待启动配置后使用
* @param  none
* @retval none
*/
static void app_mipi_rx_start_cb(void *data)
{
    /* RX start */
    hal_dsi_rx_ctrl_start(g_rx_ctrl_handle);
    /* close cb */
    hal_gpio_set_ap_reset_int(DISABLE, NULL, DETECT_HIGH_LVL);
    TAU_LOGD("rx start1\n");
}
#endif

/**
* @brief  mipi tx 启动
* @param  none
* @retval none
*/
static void app_mipi_tx_start(void)
{
    /* Init panel */
    app_init_panel();
#if 0//AP_SWIRE_OUTPUT
    hal_swire_enable(true);
    hal_swire_set_pulse(41);
#endif
    
    /* TX start */
    hal_dsi_tx_ctrl_start( g_tx_ctrl_handle );
    
#if RX_WAIT_TEAR_ON
    te_mode_e default_te = sg_ap_set_tear_on ? TE_60HZ_MODE : TE_USER_MODE;
    hal_dsi_tx_ctrl_set_tear_mode(g_tx_ctrl_handle, default_te);
#else
//    hal_dsi_tx_ctrl_set_tear_mode( g_tx_ctrl_handle, TE_60HZ_MODE );
//	hal_dsi_tx_ctrl_set_tear_mode( g_tx_ctrl_handle, TE_120HZ_MODE ); // IQOO10Pro 在这之前就发送了120Hz固定帧率

#endif
    
    panel_display_done = true;
    /* Display on */
    delayMs( 20 );
    hal_dsi_tx_ctrl_write_cmd( 0x05, 0, 1, 0x29 );

#if AP_SWIRE_OUTPUT
    hal_swire_set_pulse(SWIRE_DEFAULT_PULSE);
#endif

#if (TX_USE_CMD_MODE & (!TX_CMD_MODE_WITHOUT_TE))
    // hal_dsi_tx_ctrl_gen_a_frame(); /* FIXME */
    app_tx_cmd_app_init_panel_te_int(IO_PAD_AP_TE, DETECT_RISING_EDGE); /* 注册屏端TE中断 */
#endif

    TAU_LOGD("tx_start \n");
}

/**
* @brief  ap rstn 拉高中断回调,用于息屏唤醒
* @param  none
* @retval none
*/
static void ap_rstn_pull_high_cb(void *data)
{
    /* system resume begin */
    sg_system_resume = true;
    /* 关闭AP reset检查     */
    hal_gpio_set_ap_reset_int(DISABLE, NULL, DETECT_HIGH_LVL);
		TAU_LOGD("sys_restart \n");
}

/**
* @brief  ap rstn 拉高中断回调,用于息屏待机
* @param  none
* @retval none
*/
static void ap_rstn_pull_down_cb(void *data)
{
    sg_system_suspend = true;
    /* 关闭AP reset检查     */
    hal_gpio_set_ap_reset_int(DISABLE, NULL, DETECT_HIGH_LVL);
    TAU_LOGD("ap_rstn_pull_down_cb\n");
}

/**
* @brief  GPIO初始化配置,根据实际原理图提前配置IO功能以及状态,默认功能可不配置
*         TP相关I2C/SPI 在tp_transfer.c,也可以挪到此函数初始化
* @param  none
* @retval none
*/
void app_gpio_init(void)
{
    io_pad_attr_t attrs[] =
    {
        {IO_PIN_8,  PIN8_MODE_GPIO7,     IO_IOE_OUTPUT, IO_LVL_LOW},/* PIN_8(TD_RSTN), GPIO,输出,低电平 */
        {IO_PIN_17, PIN17_MODE_GPIO8,   IO_IOE_OUTPUT, IO_LVL_LOW}, /* PIN_17(PWMEN), GPIO,输出,低电平 */
        {IO_PIN_16, PIN16_MODE_GPIO2,   IO_IOE_OUTPUT, IO_LVL_LOW}, /* PIN_16(AP_INT),GPIO,输出,低电平 */
        {IO_PIN_15, PIN15_MODE_GPIO4,   IO_IOE_OUTPUT, IO_LVL_LOW}, /* PIN_15(AP_SWIRE),GPIO,输出,低电平 */
#if (TX_USE_CMD_MODE & (!TX_CMD_MODE_WITHOUT_TE))
        {IO_PIN_29, PIN29_MODE_GPIO3,   IO_IOE_INPUT, IO_LVL_NONE}, /* PIN_29(AP_TE), GPIO,输入 */
#else
        {IO_PIN_29, PIN29_MODE_TEAR,    IO_IOE_NONE, IO_LVL_NONE},  /* PIN_29(AP_TE), 硬件TEAR输出模式 */
#endif
    };
    uint8_t size = sizeof(attrs) / sizeof(io_pad_attr_t);
    hal_gpio_config_pad(attrs, size);
}

/**
* @brief  显示相关模块初始化,包括MIPI RX/TX/PWM/SWIRE/GPIO等
* @param  none
* @retval none
*/
void app_display_init(void)
{
    /* mipi rx初始化 */
    app_mipi_rx_init();

    /* VCC 主供电,等待VCC Power Ready,此时RX初始化完成可以响应MIPI命令 */
    if (MAIN_POWER_SELECT == PWR_SEL_VCC)
    {
        while (hal_pwr_get_vcc_power_ready() == false);
    }

    /* GPIO 初始化   */
    app_gpio_init();

    /* 背光初始化      */
#if AP_SWIRE_OUTPUT
    hal_swire_init();   /*  swire init */
    hal_swire_set_timer(TIMER_NUM0, 16, true); /* swire连续发送,绑定timer进行发送 */
#endif
#if ANALOG_PWM_OUTPUT
    if (hal_pwm_init(PWM_FREQUENCY, PWM_DUTY_STEP))
    {
        hal_pwm_enable(true);
        hal_pwm_set_elvcc_output(true);
    }
#endif

    /* mipi tx 初始化*/
    app_mipi_tx_init();

#if TX_START_AFTER_APRST
    TAU_LOGD("wait exit sleep mode\n");
#else
    app_mipi_tx_start();
#endif
}

/**
* @brief  系统resume
* @param  sleep_mode: sleep 模式
* @retval none
*/
static void app_system_resume(pwr_sleep_mode_e sleep_mode)
{
    /* 退出sleep mode, 电源切换 */
    hal_pwr_exit_sleep_mode();

    /* display resume */
    app_display_init();

#if TOUCH_ENABLE
    /* touch resume,注意,stop sleep mode 退出时需要对注册的唤醒引脚重新初始化 */
    app_tp_write_other_operations(NULL, 0);
#endif
    TAU_LOGD("system resume\n");
}

/**
* @brief  系统suspend,进入sleep mode
* @param  sleep_mode: sleep 模式
*         常见无法待机/待机重启问题分析步骤:
*         1.确定待机电源(切换电源是否正常)
*         2.确认待机时序,比如主供电为VCI时,VCI掉电前已经进入sleep mode,完成电源切换
*         3.确定唤醒源是稳定的电平状态,不存在虚焊/上下拉电源异常等情况
*         4.打开SLEEP_MODE_DEBUG 的宏(关闭TP触摸报点、关闭部分打印)进行调试,检查'system reset'打印以及换成flag
*         5.检查PCB layout 是否符合规范,示波器抓取13CP/VCI 波形,1.8V主供电时13CP为1.8V,VCI主供电时13CP为1.5V,进入sleep mode电源切换时13CP电平会变化，
*           检查模组切换过程13CP的电平切换是否平缓，如果出现大幅度向下的脉冲并且低于1.2V时会出现重启或者hardfault的情况,这种情况均为PCB layout时电源回路不符合规范
* @retval none
*/
static void app_system_suspend(pwr_sleep_mode_e sleep_mode)
{
    /* 关闭图像通路 */
    hal_dsi_tx_ctrl_stop(g_tx_ctrl_handle);
    hal_dsi_tx_ctrl_deinit(g_tx_ctrl_handle);
    hal_dsi_rx_ctrl_stop(g_rx_ctrl_handle);
    hal_dsi_rx_ctrl_deinit(g_rx_ctrl_handle);

    /* Tear拉低 */
    hal_gpio_init_output(IO_PAD_AP_TE, IO_LVL_LOW);
    panel_display_done = false;
#if RX_WAIT_TEAR_ON
    sg_ap_set_tear_on = false;
#endif

    /* 关闭外设 比如Swire/I2C/Flash 等 */
#if AP_SWIRE_OUTPUT
    hal_swire_deinit();
#endif
#if ANALOG_PWM_OUTPUT
    hal_pwm_deinit();
#endif

    /* 关闭共享flash */
#if SHARE_FLASH_ENABLE
    hal_flash_share_mode(false);
#endif

    /* 配置待机供电电源 */
    hal_pwr_set_sleep_mode_power(SLEEP_MODE_POWER);

#if SLEEP_MODE_DEBUG
    TAU_LOGD("s %d\n", sleep_mode);
#endif

    if (sleep_mode == PWR_NORMAL_SLEEP_MODE)
    {
        /* normal sleep mode, MCU可以正常工作 */
        hal_gpio_set_ap_reset_int(ENABLE, ap_rstn_pull_high_cb, DETECT_HIGH_LVL);
        /* 函数立即返回,外设跟MCU正常运行,可根据实际情况如TP通讯结果等,再进入stop/deep sleep mode */
				hal_pwr_set_pvd(PVD_VCI,true);
        hal_pwr_enter_normal_sleep_mode();
				TAU_LOGD("normal sleep mode\n");
    }
    else if (sleep_mode == PWR_STOP_SLEEP_MODE)
    {
        /* stop sleep mode, MCU停止工作,唤醒后程序继续运行 */

        /* 1.注册对应 wakeup IO */
        hal_pwr_set_stop_sleep_wakeup_pin(IO_PAD_AP_RSTN, WUP_RISING_EDGE);             /* 注册AP RSTN 上升沿唤醒*/
        //hal_pwr_set_stop_sleep_wakeup_pin(IO_PAD_AP_SPIS_CSN, WUP_FALLING_EDGE);      /* 注册SPIS CS引脚 下升沿唤醒*/
        //hal_pwr_set_stop_sleep_wakeup_pin(IO_PAD_TD_INT, WUP_FALLING_EDGE);           /* 注册TD_INT引脚 下升沿唤醒*/

        /* 2.进入stop sleep mode, PC指针停止,知道注册的唤醒IO唤醒后才会退出函数,返回值为对应的IO */
        io_pad_e wakeup_io = hal_pwr_enter_stop_sleep_mode();
        if (wakeup_io == IO_PAD_AP_RSTN)
        {
            sg_system_resume = true;
        }
        else
        {
            /* Not impletmented,应用根据不同的唤醒引脚执行不同操作,也可以重新注册唤醒引脚重新进入sleep mode */
            TAU_LOGD("wakeup_io %d FIXME touch wakeup convert to AP\n", wakeup_io);
            hal_gpio_set_ap_reset_int(ENABLE, ap_rstn_pull_high_cb, DETECT_HIGH_LVL);
        }
    }
    else
    {
        /* deep sleep mode, 配置AP RSTN 上升沿,TD INT 下降沿唤醒, 唤醒后芯片重启,可通过hal_pwr_get_reset_flag确定唤醒原因  */
        hal_pwr_enter_deep_sleep_mode(WUP_RISING_EDGE, WUP_FALLING_EDGE);
    }

}

/**
* @brief  系统process处理函数,处理待机唤醒等
* @param  none
* @retval none
*/
static void app_system_process(void)
{
    if (sg_system_suspend)
    {
        /* 系统进入sleep mode */
        app_system_suspend(SLEEP_MODE_SELECT);
        sg_system_suspend = false;
    }

    if (sg_system_resume)
    {
        /* 系统退出sleep mode */
        app_system_resume(SLEEP_MODE_SELECT);
        sg_system_resume = false;
    }

#if TX_START_AFTER_APRST
    if (sg_tx_start_in_process)
    {
        app_mipi_tx_start();
        sg_tx_start_in_process = false;
    }
#endif
}

/* 通话熄屏功能 */ 
// 注意，函数在主循环函数中调用，board Init 中打开 systick定时器为1ms，本机制是识别高电平变化进入熄屏处理流程，其它触发形式按实际触摸规定来。熄屏管脚若开机时刻异常触发，考虑加上拉或下拉电阻！！！

static void PhoneDisOffDeal( void )
{
    static unsigned char Flag = 0;       // 状态标志，防止反复执行函数状态
    static unsigned int  ExitDelay = 0;
    
    if( 0 ==  Flag )
    {
        /* 引脚初始化 */
        hal_gpio_set_pull_state( TOUCH_PHONE_DIS_OFF_PIN, ENABLE, DISABLE ); // 设定初始上拉状态
        hal_gpio_init_input( TOUCH_PHONE_DIS_OFF_PIN );
        Flag = 1; // 更新标志，防止反复执行, 并进入到下个状态
    }
    else if( ( IO_LVL_HIGH == hal_gpio_get_input_data( TOUCH_PHONE_DIS_OFF_PIN ) ) && ( 1 == Flag ) )
    {
        /* 高电平，刷黑操作 */
        hal_dsi_tx_ctrl_write_cmd( 0x05, 0, 1, 0x22 );
        Flag = 2; // 更新标志，防止反复执行        
    }
    else if( ( IO_LVL_LOW == hal_gpio_get_input_data( TOUCH_PHONE_DIS_OFF_PIN ) ) && ( 2 == Flag ) )
    {
        /* 低电平, 退出操作 */
        ExitDelay = hal_system_get_tick(); // 更新计时时间
        //hal_dsi_tx_ctrl_write_cmd( 0x05, 0, 1, 0x13 );
        //Flag = 1; // 更新标志，防止反复执行
        Flag = 3;
    }
    else if( 3 == Flag )
    {
        if( hal_system_get_tick() - ExitDelay > 1000 ) // 延时 1S 退出黑屏状态
        {
            hal_dsi_tx_ctrl_write_cmd( 0x05, 0, 1, 0x13 );
            Flag = 1; // 更新标志，防止反复执行
        }
        else if( IO_LVL_LOW == hal_gpio_get_input_data( TOUCH_PHONE_DIS_OFF_PIN ) ) // 倒计时阶段又挡住了屏幕，继续更新时间
        {
            ExitDelay = hal_system_get_tick(); // 更新计时时间
        }
    }

}
/**
* @brief  S8 demo 主函数
* @param  none
* @retval none
*/
void s8_demo(void)
{
#if SLEEP_MODE_DEBUG
    TAU_LOGD("system reset %d\n", hal_pwr_get_reset_flag());
#endif
//    TAU_LOGD("x90Pro start \n");
    /* 电源选择,上电只需要选择一次 */
    hal_pwr_set_main_power(MAIN_POWER_SELECT);        /* 切换供电*/

    /* 显示模块初始 */
    app_display_init();

    /* touch 相关模块初始化 */
#if TOUCH_ENABLE
    /* TP 初始化 */
    app_tp_init();
    app_tp_phone_clear_reset_on();
    /* 与屏的TP 模块通讯并初始化 */
    app_tp_transfer_screen_start();
#endif

    TAU_LOGD("x90Pro init done \n");

#if TEST_TX_VPG
    hal_dsi_tx_ctrl_set_vpg(ENABLE, TX_VPG_V_COLOR, false);
    while(1);
#endif

    while (1)
    {
#if TOUCH_ENABLE
        /* 等待屏TP中断上报做TP协议转换,注意接口实现不可阻塞!否则会影响sleep mode */
#if !SLEEP_MODE_DEBUG
        app_tp_transfer_screen_int();
#endif
#endif
				/* 通话息屏功能处理 */
				PhoneDisOffDeal();
			
        /* DCS 命令异步处理 */
        while (hal_dsi_rx_ctrl_dcs_async_handler(g_rx_ctrl_handle));

        /* 系统事件处理(sleep mode) */
        app_system_process();
    }
}
#endif
